@@ -20,45 +20,43 @@ package org.apache.catalina.authenticator;
 
 
 import java.io.IOException;
-import java.security.MessageDigest;
-import java.security.NoSuchAlgorithmException;
 import java.security.Principal;
+import java.security.cert.X509Certificate;
 import java.text.SimpleDateFormat;
 import java.util.Date;
 import java.util.Locale;
-import java.util.Random;
 
 import javax.servlet.ServletException;
 import javax.servlet.http.Cookie;
+import javax.servlet.http.HttpServletResponse;
 
 import org.apache.catalina.Authenticator;
 import org.apache.catalina.Container;
 import org.apache.catalina.Context;
-import org.apache.catalina.Lifecycle;
+import org.apache.catalina.Globals;
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.LifecycleListener;
 import org.apache.catalina.Manager;
-import org.apache.catalina.Pipeline;
 import org.apache.catalina.Realm;
 import org.apache.catalina.Session;
 import org.apache.catalina.Valve;
+import org.apache.catalina.Wrapper;
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.connector.Response;
 import org.apache.catalina.deploy.LoginConfig;
 import org.apache.catalina.deploy.SecurityConstraint;
 import org.apache.catalina.util.DateTool;
-import org.apache.catalina.util.LifecycleSupport;
-import org.apache.catalina.util.StringManager;
+import org.apache.catalina.util.SessionIdGenerator;
 import org.apache.catalina.valves.ValveBase;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.res.StringManager;
 
 
 /**
  * Basic implementation of the <b>Valve</b> interface that enforces the
  * <code>&lt;security-constraint&gt;</code> elements in the web application
  * deployment descriptor.  This functionality is implemented as a Valve
- * so that it can be ommitted in environments that do not require these
+ * so that it can be omitted in environments that do not require these
  * features.  Individual implementations of each supported authentication
  * method can subclass this base class as required.
  * <p>
@@ -71,32 +69,28 @@ import org.apache.juli.logging.LogFactory;
  * requests.  Requests of any other type will simply be passed through.
  *
  * @author Craig R. McClanahan
- * @version $Id: AuthenticatorBase.java 993409 2010-09-07 16:05:25Z markt $
+ * @version $Id: AuthenticatorBase.java 1126273 2011-05-22 22:54:15Z markt $
  */
 
 
-public abstract class AuthenticatorBase
-    extends ValveBase
-    implements Authenticator, Lifecycle {
-    private static Log log = LogFactory.getLog(AuthenticatorBase.class);
+public abstract class AuthenticatorBase extends ValveBase
+        implements Authenticator {
 
+    private static final Log log = LogFactory.getLog(AuthenticatorBase.class);
+
+
+    //------------------------------------------------------ Constructor
+    public AuthenticatorBase() {
+        super(true);
+    }
 
     // ----------------------------------------------------- Instance Variables
 
 
     /**
-     * The default message digest algorithm to use if we cannot use
-     * the requested one.
+     * Authentication header
      */
-    protected static final String DEFAULT_ALGORITHM = "MD5";
-
-
-    /**
-     * The number of random bytes to include when generating a
-     * session identifier.
-     */
-    protected static final int SESSION_ID_BYTES = 16;
-
+    protected static final String AUTH_HEADER_NAME = "WWW-Authenticate";
 
     /**
      * Default authentication realm name.
@@ -104,11 +98,16 @@ public abstract class AuthenticatorBase
     protected static final String REALM_NAME = "Authentication required";
 
     /**
-     * The message digest algorithm to be used when generating session
-     * identifiers.  This must be an algorithm supported by the
-     * <code>java.security.MessageDigest</code> class on your platform.
+     * Should a session always be used once a user is authenticated? This may
+     * offer some performance benefits since the session can then be used to
+     * cache the authenticated Principal, hence removing the need to
+     * authenticate the user via the Realm on every request. This may be of help
+     * for combinations such as BASIC authentication used with the JNDIRealm or
+     * DataSourceRealms. However there will also be the performance cost of
+     * creating and GC'ing the session. By default, a session will not be
+     * created. 
      */
-    protected String algorithm = DEFAULT_ALGORITHM;
+    protected boolean alwaysUseSession = false;
 
 
     /**
@@ -130,20 +129,6 @@ public abstract class AuthenticatorBase
     protected Context context = null;
 
 
-    /**
-     * Return the MessageDigest implementation to be used when
-     * creating session identifiers.
-     */
-    protected MessageDigest digest = null;
-
-
-    /**
-     * A String initialization parameter used to increase the entropy of
-     * the initialization of our random number generator.
-     */
-    protected String entropy = null;
-
-
     /**
      * Descriptive information about this implementation.
      */
@@ -158,28 +143,41 @@ public abstract class AuthenticatorBase
 
     /**
      * Flag to determine if we disable proxy caching with headers incompatible
-     * with IE 
+     * with IE.
      */
-    protected boolean securePagesWithPragma = true;
+    protected boolean securePagesWithPragma = false;
     
     /**
-     * The lifecycle event support for this component.
+     * The Java class name of the secure random number generator class to be
+     * used when generating SSO session identifiers. The random number generator
+     * class must be self-seeding and have a zero-argument constructor. If not
+     * specified, an instance of {@link java.security.SecureRandom} will be
+     * generated.
      */
-    protected LifecycleSupport lifecycle = new LifecycleSupport(this);
-
+    protected String secureRandomClass = null;
 
     /**
-     * A random number generator to use when generating session identifiers.
+     * The name of the algorithm to use to create instances of
+     * {@link java.security.SecureRandom} which are used to generate SSO session
+     * IDs. If no algorithm is specified, SHA1PRNG is used. To use the platform
+     * default (which may be SHA1PRNG), specify the empty string. If an invalid
+     * algorithm and/or provider is specified the SecureRandom instances will be
+     * created using the defaults. If that fails, the SecureRandom instances
+     * will be created using platform defaults.
      */
-    protected Random random = null;
-
+    protected String secureRandomAlgorithm = "SHA1PRNG";
 
     /**
-     * The Java class name of the random number generator class to be used
-     * when generating session identifiers.
+     * The name of the provider to use to create instances of
+     * {@link java.security.SecureRandom} which are used to generate session SSO
+     * IDs. If no algorithm is specified the of SHA1PRNG default is used. If an
+     * invalid algorithm and/or provider is specified the SecureRandom instances
+     * will be created using the defaults. If that fails, the SecureRandom
+     * instances will be created using platform defaults.
      */
-    protected String randomClass = "java.security.SecureRandom";
+    protected String secureRandomProvider = null;
 
+    protected SessionIdGenerator sessionIdGenerator = null;
 
     /**
      * The string manager for this package.
@@ -195,12 +193,6 @@ public abstract class AuthenticatorBase
     protected SingleSignOn sso = null;
 
 
-    /**
-     * Has this component been started?
-     */
-    protected boolean started = false;
-
-
     /**
      * "Expires" header always set to Date(1), so generate once only
      */
@@ -212,28 +204,6 @@ public abstract class AuthenticatorBase
     // ------------------------------------------------------------- Properties
 
 
-    /**
-     * Return the message digest algorithm for this Manager.
-     */
-    public String getAlgorithm() {
-
-        return (this.algorithm);
-
-    }
-
-
-    /**
-     * Set the message digest algorithm for this Manager.
-     *
-     * @param algorithm The new message digest algorithm
-     */
-    public void setAlgorithm(String algorithm) {
-
-        this.algorithm = algorithm;
-
-    }
-
-
     /**
      * Return the cache authenticated Principals flag.
      */
@@ -259,6 +229,7 @@ public abstract class AuthenticatorBase
     /**
      * Return the Container to which this Valve is attached.
      */
+    @Override
     public Container getContainer() {
 
         return (this.context);
@@ -271,9 +242,10 @@ public abstract class AuthenticatorBase
      *
      * @param container The container to which we are attached
      */
+    @Override
     public void setContainer(Container container) {
 
-        if (!(container instanceof Context))
+        if (container != null && !(container instanceof Context))
             throw new IllegalArgumentException
                 (sm.getString("authenticator.notContext"));
 
@@ -283,36 +255,10 @@ public abstract class AuthenticatorBase
     }
 
 
-    /**
-     * Return the entropy increaser value, or compute a semi-useful value
-     * if this String has not yet been set.
-     */
-    public String getEntropy() {
-
-        // Calculate a semi-useful value if this has not been set
-        if (this.entropy == null)
-            setEntropy(this.toString());
-
-        return (this.entropy);
-
-    }
-
-
-    /**
-     * Set the entropy increaser value.
-     *
-     * @param entropy The new entropy increaser value
-     */
-    public void setEntropy(String entropy) {
-
-        this.entropy = entropy;
-
-    }
-
-
     /**
      * Return descriptive information about this Valve implementation.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -320,27 +266,6 @@ public abstract class AuthenticatorBase
     }
 
 
-    /**
-     * Return the random number generator class name.
-     */
-    public String getRandomClass() {
-
-        return (this.randomClass);
-
-    }
-
-
-    /**
-     * Set the random number generator class name.
-     *
-     * @param randomClass The new random number generator class name
-     */
-    public void setRandomClass(String randomClass) {
-
-        this.randomClass = randomClass;
-
-    }
-
     /**
      * Return the flag that states if we add headers to disable caching by
      * proxies.
@@ -403,6 +328,66 @@ public abstract class AuthenticatorBase
         this.changeSessionIdOnAuthentication = changeSessionIdOnAuthentication;
     }
 
+    /**
+     * Return the secure random number generator class name.
+     */
+    public String getSecureRandomClass() {
+
+        return (this.secureRandomClass);
+
+    }
+
+
+    /**
+     * Set the secure random number generator class name.
+     *
+     * @param secureRandomClass The new secure random number generator class
+     *                          name
+     */
+    public void setSecureRandomClass(String secureRandomClass) {
+        this.secureRandomClass = secureRandomClass;
+    }
+
+
+    /**
+     * Return the secure random number generator algorithm name.
+     */
+    public String getSecureRandomAlgorithm() {
+        return secureRandomAlgorithm;
+    }
+
+
+    /**
+     * Set the secure random number generator algorithm name.
+     *
+     * @param secureRandomAlgorithm The new secure random number generator
+     *                              algorithm name
+     */
+    public void setSecureRandomAlgorithm(String secureRandomAlgorithm) {
+        this.secureRandomAlgorithm = secureRandomAlgorithm;
+    }
+
+
+    /**
+     * Return the secure random number generator provider name.
+     */
+    public String getSecureRandomProvider() {
+        return secureRandomProvider;
+    }
+
+
+    /**
+     * Set the secure random number generator provider name.
+     *
+     * @param secureRandomProvider The new secure random number generator
+     *                             provider name
+     */
+    public void setSecureRandomProvider(String secureRandomProvider) {
+        this.secureRandomProvider = secureRandomProvider;
+    }
+
+
+
     // --------------------------------------------------------- Public Methods
 
 
@@ -416,6 +401,7 @@ public abstract class AuthenticatorBase
      * @exception IOException if an input/output error occurs
      * @exception ServletException if thrown by a processing element
      */
+    @Override
     public void invoke(Request request, Response response)
         throws IOException, ServletException {
 
@@ -458,13 +444,19 @@ public abstract class AuthenticatorBase
             }
         }
 
+        // The Servlet may specify security constraints through annotations.
+        // Ensure that they have been processed before constraints are checked
+        Wrapper wrapper = (Wrapper) request.getMappingData().wrapper;
+        if (wrapper != null) {
+            wrapper.servletSecurityAnnotationScan();
+        }
+
         Realm realm = this.context.getRealm();
         // Is this request URI subject to a security constraint?
         SecurityConstraint [] constraints
             = realm.findSecurityConstraints(request, this.context);
        
-        if ((constraints == null) /* &&
-            (!Constants.FORM_METHOD.equals(config.getAuthMethod())) */ ) {
+        if (constraints == null && !context.getPreemptiveAuthentication()) {
             if (log.isDebugEnabled())
                 log.debug(" Not subject to any constraint");
             getNext().invoke(request, response);
@@ -473,15 +465,11 @@ public abstract class AuthenticatorBase
 
         // Make sure that constrained resources are not cached by web proxies
         // or browsers as caching can provide a security hole
-        if (disableProxyCaching && 
-            // FIXME: Disabled for Mozilla FORM support over SSL 
-            // (improper caching issue)
-            //!request.isSecure() &&
+        if (constraints != null && disableProxyCaching && 
+            !request.isSecure() &&
             !"POST".equalsIgnoreCase(request.getMethod())) {
             if (securePagesWithPragma) {
-                // FIXME: These cause problems with downloading office docs
-                // from IE under SSL and may not be needed for newer Mozilla
-                // clients.
+                // Note: These can cause problems with downloading files with IE
                 response.setHeader("Pragma", "No-cache");
                 response.setHeader("Cache-Control", "no-cache");
             } else {
@@ -491,36 +479,55 @@ public abstract class AuthenticatorBase
         }
 
         int i;
-        // Enforce any user data constraint for this security constraint
-        if (log.isDebugEnabled()) {
-            log.debug(" Calling hasUserDataPermission()");
-        }
-        if (!realm.hasUserDataPermission(request, response,
-                                         constraints)) {
+        if (constraints != null) {
+            // Enforce any user data constraint for this security constraint
             if (log.isDebugEnabled()) {
-                log.debug(" Failed hasUserDataPermission() test");
+                log.debug(" Calling hasUserDataPermission()");
+            }
+            if (!realm.hasUserDataPermission(request, response,
+                                             constraints)) {
+                if (log.isDebugEnabled()) {
+                    log.debug(" Failed hasUserDataPermission() test");
+                }
+                /*
+                 * ASSERT: Authenticator already set the appropriate
+                 * HTTP status code, so we do not have to do anything special
+                 */
+                return;
             }
-            /*
-             * ASSERT: Authenticator already set the appropriate
-             * HTTP status code, so we do not have to do anything special
-             */
-            return;
         }
 
         // Since authenticate modifies the response on failure,
         // we have to check for allow-from-all first.
-        boolean authRequired = true;
-        for(i=0; i < constraints.length && authRequired; i++) {
-            if(!constraints[i].getAuthConstraint()) {
-                authRequired = false;
-            } else if(!constraints[i].getAllRoles()) {
-                String [] roles = constraints[i].findAuthRoles();
-                if(roles == null || roles.length == 0) {
+        boolean authRequired;
+        if (constraints == null) {
+            authRequired = false;
+        } else {
+            authRequired = true;
+            for(i=0; i < constraints.length && authRequired; i++) {
+                if(!constraints[i].getAuthConstraint()) {
                     authRequired = false;
+                } else if(!constraints[i].getAllRoles()) {
+                    String [] roles = constraints[i].findAuthRoles();
+                    if(roles == null || roles.length == 0) {
+                        authRequired = false;
+                    }
                 }
             }
         }
-             
+
+        if (!authRequired) {
+            authRequired =
+                request.getCoyoteRequest().getMimeHeaders().getValue(
+                        "authorization") != null;
+        }
+
+        if (!authRequired) {
+            X509Certificate[] certs = (X509Certificate[]) request.getAttribute(
+                    Globals.CERTIFICATES_ATTR);
+            authRequired = certs != null && certs.length > 0;
+        }
+
         if(authRequired) {  
             if (log.isDebugEnabled()) {
                 log.debug(" Calling authenticate()");
@@ -539,21 +546,23 @@ public abstract class AuthenticatorBase
             
         }
     
-        if (log.isDebugEnabled()) {
-            log.debug(" Calling accessControl()");
-        }
-        if (!realm.hasResourcePermission(request, response,
-                                         constraints,
-                                         this.context)) {
+        if (constraints != null) {
             if (log.isDebugEnabled()) {
-                log.debug(" Failed accessControl() test");
+                log.debug(" Calling accessControl()");
+            }
+            if (!realm.hasResourcePermission(request, response,
+                                             constraints,
+                                             this.context)) {
+                if (log.isDebugEnabled()) {
+                    log.debug(" Failed accessControl() test");
+                }
+                /*
+                 * ASSERT: AccessControl method has already set the
+                 * appropriate HTTP status code, so we do not have to do
+                 * anything special
+                 */
+                return;
             }
-            /*
-             * ASSERT: AccessControl method has already set the
-             * appropriate HTTP status code, so we do not have to do
-             * anything special
-             */
-            return;
         }
     
         // Any and all specified constraints have been satisfied
@@ -568,8 +577,6 @@ public abstract class AuthenticatorBase
     // ------------------------------------------------------ Protected Methods
 
 
-
-
     /**
      * Associate the specified single sign on identifier with the
      * specified Session.
@@ -593,100 +600,19 @@ public abstract class AuthenticatorBase
      * created a response challenge already.
      *
      * @param request Request we are processing
-     * @param response Response we are creating
+     * @param response Response we are populating
      * @param config    Login configuration describing how authentication
      *              should be performed
      *
      * @exception IOException if an input/output error occurs
      */
-    protected abstract boolean authenticate(Request request,
-                                            Response response,
+    @Override
+    public abstract boolean authenticate(Request request,
+                                            HttpServletResponse response,
                                             LoginConfig config)
         throws IOException;
 
 
-    /**
-     * Generate and return a new session identifier for the cookie that
-     * identifies an SSO principal.
-     */
-    protected synchronized String generateSessionId() {
-
-        // Generate a byte array containing a session identifier
-        byte bytes[] = new byte[SESSION_ID_BYTES];
-        getRandom().nextBytes(bytes);
-        bytes = getDigest().digest(bytes);
-
-        // Render the result as a String of hexadecimal digits
-        StringBuffer result = new StringBuffer();
-        for (int i = 0; i < bytes.length; i++) {
-            byte b1 = (byte) ((bytes[i] & 0xf0) >> 4);
-            byte b2 = (byte) (bytes[i] & 0x0f);
-            if (b1 < 10)
-                result.append((char) ('0' + b1));
-            else
-                result.append((char) ('A' + (b1 - 10)));
-            if (b2 < 10)
-                result.append((char) ('0' + b2));
-            else
-                result.append((char) ('A' + (b2 - 10)));
-        }
-        return (result.toString());
-
-    }
-
-
-    /**
-     * Return the MessageDigest object to be used for calculating
-     * session identifiers.  If none has been created yet, initialize
-     * one the first time this method is called.
-     */
-    protected synchronized MessageDigest getDigest() {
-
-        if (this.digest == null) {
-            try {
-                this.digest = MessageDigest.getInstance(algorithm);
-            } catch (NoSuchAlgorithmException e) {
-                try {
-                    this.digest = MessageDigest.getInstance(DEFAULT_ALGORITHM);
-                } catch (NoSuchAlgorithmException f) {
-                    this.digest = null;
-                }
-            }
-        }
-
-        return (this.digest);
-
-    }
-
-
-    /**
-     * Return the random number generator instance we should use for
-     * generating session identifiers.  If there is no such generator
-     * currently defined, construct and seed a new one.
-     */
-    protected synchronized Random getRandom() {
-
-        if (this.random == null) {
-            try {
-                Class clazz = Class.forName(randomClass);
-                this.random = (Random) clazz.newInstance();
-                long seed = System.currentTimeMillis();
-                char entropy[] = getEntropy().toCharArray();
-                for (int i = 0; i < entropy.length; i++) {
-                    long update = ((byte) entropy[i]) << ((i % 8) * 8);
-                    seed ^= update;
-                }
-                this.random.setSeed(seed);
-            } catch (Exception e) {
-                this.random = new java.util.Random();
-            }
-        }
-
-        return (this.random);
-
-    }
-
-
     /**
      * Attempts reauthentication to the <code>Realm</code> using
      * the credentials included in argument <code>entry</code>.
@@ -737,13 +663,15 @@ public abstract class AuthenticatorBase
      * @param username Username used to authenticate (if any)
      * @param password Password used to authenticate (if any)
      */
-    protected void register(Request request, Response response,
+    public void register(Request request, HttpServletResponse response,
                             Principal principal, String authType,
                             String username, String password) {
 
-        if (log.isDebugEnabled())
-            log.debug("Authenticated '" + principal.getName() + "' with type '"
-                + authType + "'");
+        if (log.isDebugEnabled()) {
+            String name = (principal == null) ? "none" : principal.getName(); 
+            log.debug("Authenticated '" + name + "' with type '" + authType +
+                    "'");
+        }
 
         // Cache the authentication information in our request
         request.setAuthType(authType);
@@ -751,10 +679,14 @@ public abstract class AuthenticatorBase
 
         Session session = request.getSessionInternal(false);
         
-        if (session != null && changeSessionIdOnAuthentication) {
-            Manager manager = request.getContext().getManager();
-            manager.changeSessionId(session);
-            request.changeSessionId(session.getId());
+        if (session != null) {
+            if (changeSessionIdOnAuthentication) {
+                Manager manager = request.getContext().getManager();
+                manager.changeSessionId(session);
+                request.changeSessionId(session.getId());
+            }
+        } else if (alwaysUseSession) {
+            session = request.getSessionInternal(true);
         }
 
         // Cache the authentication information in our session, if any
@@ -783,7 +715,7 @@ public abstract class AuthenticatorBase
         String ssoId = (String) request.getNote(Constants.REQ_SSOID_NOTE);
         if (ssoId == null) {
             // Construct a cookie to be returned to the client
-            ssoId = generateSessionId();
+            ssoId = sessionIdGenerator.generateSessionId();
             Cookie cookie = new Cookie(Constants.SINGLE_SIGN_ON_COOKIE, ssoId);
             cookie.setMaxAge(-1);
             cookie.setPath("/");
@@ -797,15 +729,27 @@ public abstract class AuthenticatorBase
                 cookie.setDomain(ssoDomain);
             }
 
-            response.addCookieInternal(cookie, context.getUseHttpOnly());
+            // Configure httpOnly on SSO cookie using same rules as session cookies
+            if (request.getServletContext().getSessionCookieConfig().isHttpOnly() ||
+                    request.getContext().getUseHttpOnly()) {
+                cookie.setHttpOnly(true);
+            }
+            
+            response.addCookie(cookie);
 
             // Register this principal with our SSO valve
             sso.register(ssoId, principal, authType, username, password);
             request.setNote(Constants.REQ_SSOID_NOTE, ssoId);
 
         } else {
-            // Update the SSO session with the latest authentication data
-            sso.update(ssoId, principal, authType, username, password);
+            if (principal == null) {
+                // Registering a programmatic logout
+                sso.deregister(ssoId);
+                return;
+            } else {
+                // Update the SSO session with the latest authentication data
+                sso.update(ssoId, principal, authType, username, password);
+            }
         }
 
         // Fix for Bug 10040
@@ -820,71 +764,56 @@ public abstract class AuthenticatorBase
 
     }
 
+    @Override
+    public void login(String username, String password, Request request)
+            throws ServletException {
+        Principal principal = doLogin(request, username, password);
+        register(request, request.getResponse(), principal,
+                    getAuthMethod(), username, password);
+    }
 
-    // ------------------------------------------------------ Lifecycle Methods
-
+    protected abstract String getAuthMethod();
 
     /**
-     * Add a lifecycle event listener to this component.
-     *
-     * @param listener The listener to add
+     * Process the login request.
+     * 
+     * @param request   Associated request
+     * @param username  The user
+     * @param password  The password
+     * @return          The authenticated Principal
+     * @throws ServletException
      */
-    public void addLifecycleListener(LifecycleListener listener) {
+    protected Principal doLogin(Request request, String username,
+            String password) throws ServletException {
+        Principal p = context.getRealm().authenticate(username, password);
+        if (p == null) {
+            throw new ServletException(sm.getString("authenticator.loginFail"));
+        }
+        return p;
+    }
 
-        lifecycle.addLifecycleListener(listener);
+    @Override
+    public void logout(Request request) throws ServletException {
+        register(request, request.getResponse(), null,
+                null, null, null);
 
     }
 
-
     /**
-     * Get the lifecycle listeners associated with this lifecycle. If this 
-     * Lifecycle has no listeners registered, a zero-length array is returned.
-     */
-    public LifecycleListener[] findLifecycleListeners() {
-
-        return lifecycle.findLifecycleListeners();
-
-    }
-
-
-    /**
-     * Remove a lifecycle event listener from this component.
-     *
-     * @param listener The listener to remove
-     */
-    public void removeLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.removeLifecycleListener(listener);
-
-    }
-
-
-    /**
-     * Prepare for the beginning of active use of the public methods of this
-     * component.  This method should be called after <code>configure()</code>,
-     * and before any of the public methods of the component are utilized.
+     * Start this component and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
      *  that prevents this component from being used
      */
-    public void start() throws LifecycleException {
-
-        // Validate and update our current component state
-        if (started)
-            throw new LifecycleException
-                (sm.getString("authenticator.alreadyStarted"));
-        lifecycle.fireLifecycleEvent(START_EVENT, null);
-        started = true;
-
+    @Override
+    protected synchronized void startInternal() throws LifecycleException {
+        
         // Look up the SingleSignOn implementation in our request processing
         // path, if there is one
         Container parent = context.getParent();
         while ((sso == null) && (parent != null)) {
-            if (!(parent instanceof Pipeline)) {
-                parent = parent.getParent();
-                continue;
-            }
-            Valve valves[] = ((Pipeline) parent).getValves();
+            Valve valves[] = parent.getPipeline().getValves();
             for (int i = 0; i < valves.length; i++) {
                 if (valves[i] instanceof SingleSignOn) {
                     sso = (SingleSignOn) valves[i];
@@ -901,29 +830,27 @@ public abstract class AuthenticatorBase
                 log.debug("No SingleSignOn Valve is present");
         }
 
+        sessionIdGenerator = new SessionIdGenerator();
+        sessionIdGenerator.setSecureRandomAlgorithm(getSecureRandomAlgorithm());
+        sessionIdGenerator.setSecureRandomClass(getSecureRandomClass());
+        sessionIdGenerator.setSecureRandomProvider(getSecureRandomProvider());
+
+        super.startInternal();
     }
 
 
     /**
-     * Gracefully terminate the active use of the public methods of this
-     * component.  This method should be the last one called on a given
-     * instance of this component.
+     * Stop this component and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
-     *  that needs to be reported
+     *  that prevents this component from being used
      */
-    public void stop() throws LifecycleException {
+    @Override
+    protected synchronized void stopInternal() throws LifecycleException {
 
-        // Validate and update our current component state
-        if (!started)
-            throw new LifecycleException
-                (sm.getString("authenticator.notStarted"));
-        lifecycle.fireLifecycleEvent(STOP_EVENT, null);
-        started = false;
+        super.stopInternal();
 
         sso = null;
-
     }
-
-
 }
