@@ -20,13 +20,18 @@ package org.apache.catalina.connector;
 
 import java.io.IOException;
 import java.io.UnsupportedEncodingException;
+import java.util.EnumSet;
+
+import javax.servlet.RequestDispatcher;
+import javax.servlet.SessionTrackingMode;
 
-import org.apache.catalina.CometEvent;
 import org.apache.catalina.Context;
-import org.apache.catalina.Globals;
 import org.apache.catalina.Host;
 import org.apache.catalina.Wrapper;
-import org.apache.catalina.util.StringManager;
+import org.apache.catalina.comet.CometEvent;
+import org.apache.catalina.comet.CometEvent.EventType;
+import org.apache.catalina.core.ApplicationSessionCookieConfig;
+import org.apache.catalina.core.AsyncContextImpl;
 import org.apache.catalina.util.ServerInfo;
 import org.apache.catalina.util.URLEncoder;
 import org.apache.coyote.ActionCode;
@@ -40,7 +45,9 @@ import org.apache.tomcat.util.buf.CharChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
 import org.apache.tomcat.util.http.Cookies;
 import org.apache.tomcat.util.http.ServerCookie;
+import org.apache.tomcat.util.net.SSLSupport;
 import org.apache.tomcat.util.net.SocketStatus;
+import org.apache.tomcat.util.res.StringManager;
 
 
 /**
@@ -49,20 +56,22 @@ import org.apache.tomcat.util.net.SocketStatus;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: CoyoteAdapter.java 1185998 2011-10-19 07:15:30Z kkolinko $
+ * @version $Id: CoyoteAdapter.java 1133401 2011-06-08 13:52:50Z markt $
  */
-
 public class CoyoteAdapter implements Adapter {
     
-    private static Log log = LogFactory.getLog(CoyoteAdapter.class);
+    private static final Log log = LogFactory.getLog(CoyoteAdapter.class);
 
     // -------------------------------------------------------------- Constants
 
-    private static final String POWERED_BY = "Servlet/2.5 JSP/2.1 " +
+    private static final String POWERED_BY = "Servlet/3.0 JSP/2.2 " +
             "(" + ServerInfo.getServerInfo() + " Java/" +
             System.getProperty("java.vm.vendor") + "/" +
             System.getProperty("java.runtime.version") + ")";
 
+    private static final EnumSet<SessionTrackingMode> SSL_ONLY =
+        EnumSet.of(SessionTrackingMode.SSL);
+
     public static final int ADAPTER_NOTES = 1;
 
 
@@ -98,7 +107,7 @@ public class CoyoteAdapter implements Adapter {
     /**
      * The string manager for this package.
      */
-    protected StringManager sm =
+    protected static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
 
@@ -132,124 +141,221 @@ public class CoyoteAdapter implements Adapter {
      * 
      * @return false to indicate an error, expected or not
      */
+    @Override
     public boolean event(org.apache.coyote.Request req, 
             org.apache.coyote.Response res, SocketStatus status) {
 
         Request request = (Request) req.getNote(ADAPTER_NOTES);
         Response response = (Response) res.getNote(ADAPTER_NOTES);
 
-        if (request.getWrapper() != null) {
+        if (request.getWrapper() == null) {
+            return false;
+        }
             
-            boolean error = false;
-            boolean read = false;
-            try {
-                if (status == SocketStatus.OPEN) {
-                    if (response.isClosed()) {
-                        // The event has been closed asynchronously, so call end instead of
-                        // read to cleanup the pipeline
-                        request.getEvent().setEventType(CometEvent.EventType.END);
-                        request.getEvent().setEventSubType(null);
-                    } else {
-                        try {
-                            // Fill the read buffer of the servlet layer
-                            if (request.read()) {
-                                read = true;
-                            }
-                        } catch (IOException e) {
-                            error = true;
-                        }
-                        if (read) {
-                            request.getEvent().setEventType(CometEvent.EventType.READ);
-                            request.getEvent().setEventSubType(null);
-                        } else if (error) {
-                            request.getEvent().setEventType(CometEvent.EventType.ERROR);
-                            request.getEvent().setEventSubType(CometEvent.EventSubType.CLIENT_DISCONNECT);
-                        } else {
-                            request.getEvent().setEventType(CometEvent.EventType.END);
-                            request.getEvent().setEventSubType(null);
-                        }
-                    }
-                } else if (status == SocketStatus.DISCONNECT) {
-                    request.getEvent().setEventType(CometEvent.EventType.ERROR);
-                    request.getEvent().setEventSubType(CometEvent.EventSubType.CLIENT_DISCONNECT);
-                    error = true;
-                } else if (status == SocketStatus.ERROR) {
-                    request.getEvent().setEventType(CometEvent.EventType.ERROR);
-                    request.getEvent().setEventSubType(CometEvent.EventSubType.IOEXCEPTION);
-                    error = true;
-                } else if (status == SocketStatus.STOP) {
+        boolean error = false;
+        boolean read = false;
+        try {
+            if (status == SocketStatus.OPEN) {
+                if (response.isClosed()) {
+                    // The event has been closed asynchronously, so call end instead of
+                    // read to cleanup the pipeline
                     request.getEvent().setEventType(CometEvent.EventType.END);
-                    request.getEvent().setEventSubType(CometEvent.EventSubType.SERVER_SHUTDOWN);
-                } else if (status == SocketStatus.TIMEOUT) {
-                    if (response.isClosed()) {
-                        // The event has been closed asynchronously, so call end instead of
-                        // read to cleanup the pipeline
-                        request.getEvent().setEventType(CometEvent.EventType.END);
-                        request.getEvent().setEventSubType(null);
-                    } else {
-                        request.getEvent().setEventType(CometEvent.EventType.ERROR);
-                        request.getEvent().setEventSubType(CometEvent.EventSubType.TIMEOUT);
-                    }
-                }
-
-                req.getRequestProcessor().setWorkerThreadName(Thread.currentThread().getName());
-                
-                // Calling the container
-                connector.getContainer().getPipeline().getFirst().event(request, response, request.getEvent());
-
-                if (!error && !response.isClosed() && (request.getAttribute(Globals.EXCEPTION_ATTR) != null)) {
-                    // An unexpected exception occurred while processing the event, so
-                    // error should be called
-                    request.getEvent().setEventType(CometEvent.EventType.ERROR);
                     request.getEvent().setEventSubType(null);
-                    error = true;
-                    connector.getContainer().getPipeline().getFirst().event(request, response, request.getEvent());
-                }
-                if (response.isClosed() || !request.isComet()) {
-                    if (status==SocketStatus.OPEN) {
-                        //CometEvent.close was called during an event.
+                } else {
+                    try {
+                        // Fill the read buffer of the servlet layer
+                        if (request.read()) {
+                            read = true;
+                        }
+                    } catch (IOException e) {
+                        error = true;
+                    }
+                    if (read) {
+                        request.getEvent().setEventType(CometEvent.EventType.READ);
+                        request.getEvent().setEventSubType(null);
+                    } else if (error) {
+                        request.getEvent().setEventType(CometEvent.EventType.ERROR);
+                        request.getEvent().setEventSubType(CometEvent.EventSubType.CLIENT_DISCONNECT);
+                    } else {
                         request.getEvent().setEventType(CometEvent.EventType.END);
                         request.getEvent().setEventSubType(null);
-                        error = true;
-                        connector.getContainer().getPipeline().getFirst().event(request, response, request.getEvent());
                     }
-                    res.action(ActionCode.ACTION_COMET_END, null);
-                } else if (!error && read && request.getAvailable()) {
-                    // If this was a read and not all bytes have been read, or if no data
-                    // was read from the connector, then it is an error
-                    request.getEvent().setEventType(CometEvent.EventType.ERROR);
-                    request.getEvent().setEventSubType(CometEvent.EventSubType.IOEXCEPTION);
-                    error = true;
-                    connector.getContainer().getPipeline().getFirst().event(request, response, request.getEvent());
-                }
-                return (!error);
-            } catch (Throwable t) {
-                if (!(t instanceof IOException)) {
-                    log.error(sm.getString("coyoteAdapter.service"), t);
                 }
+            } else if (status == SocketStatus.DISCONNECT) {
+                request.getEvent().setEventType(CometEvent.EventType.ERROR);
+                request.getEvent().setEventSubType(CometEvent.EventSubType.CLIENT_DISCONNECT);
                 error = true;
-                return false;
-            } finally {
-                req.getRequestProcessor().setWorkerThreadName(null);
-                // Recycle the wrapper request and response
-                if (error || response.isClosed() || !request.isComet()) {
-                    request.recycle();
-                    request.setFilterChain(null);
-                    response.recycle();
+            } else if (status == SocketStatus.ERROR) {
+                request.getEvent().setEventType(CometEvent.EventType.ERROR);
+                request.getEvent().setEventSubType(CometEvent.EventSubType.IOEXCEPTION);
+                error = true;
+            } else if (status == SocketStatus.STOP) {
+                request.getEvent().setEventType(CometEvent.EventType.END);
+                request.getEvent().setEventSubType(CometEvent.EventSubType.SERVER_SHUTDOWN);
+            } else if (status == SocketStatus.TIMEOUT) {
+                if (response.isClosed()) {
+                    // The event has been closed asynchronously, so call end instead of
+                    // read to cleanup the pipeline
+                    request.getEvent().setEventType(CometEvent.EventType.END);
+                    request.getEvent().setEventSubType(null);
+                } else {
+                    request.getEvent().setEventType(CometEvent.EventType.ERROR);
+                    request.getEvent().setEventSubType(CometEvent.EventSubType.TIMEOUT);
                 }
             }
+
+            req.getRequestProcessor().setWorkerThreadName(Thread.currentThread().getName());
             
-        } else {
+            // Calling the container
+            connector.getService().getContainer().getPipeline().getFirst().event(request, response, request.getEvent());
+
+            if (!error && !response.isClosed() && (request.getAttribute(
+                    RequestDispatcher.ERROR_EXCEPTION) != null)) {
+                // An unexpected exception occurred while processing the event, so
+                // error should be called
+                request.getEvent().setEventType(CometEvent.EventType.ERROR);
+                request.getEvent().setEventSubType(null);
+                error = true;
+                connector.getService().getContainer().getPipeline().getFirst().event(request, response, request.getEvent());
+            }
+            if (response.isClosed() || !request.isComet()) {
+                if (status==SocketStatus.OPEN &&
+                        request.getEvent().getEventType() != EventType.END) {
+                    //CometEvent.close was called during an event other than END
+                    request.getEvent().setEventType(CometEvent.EventType.END);
+                    request.getEvent().setEventSubType(null);
+                    error = true;
+                    connector.getService().getContainer().getPipeline().getFirst().event(request, response, request.getEvent());
+                }
+                res.action(ActionCode.COMET_END, null);
+            } else if (!error && read && request.getAvailable()) {
+                // If this was a read and not all bytes have been read, or if no data
+                // was read from the connector, then it is an error
+                request.getEvent().setEventType(CometEvent.EventType.ERROR);
+                request.getEvent().setEventSubType(CometEvent.EventSubType.IOEXCEPTION);
+                error = true;
+                connector.getService().getContainer().getPipeline().getFirst().event(request, response, request.getEvent());
+            }
+            return (!error);
+        } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
+            if (!(t instanceof IOException)) {
+                log.error(sm.getString("coyoteAdapter.service"), t);
+            }
+            error = true;
             return false;
+        } finally {
+            req.getRequestProcessor().setWorkerThreadName(null);
+            // Recycle the wrapper request and response
+            if (error || response.isClosed() || !request.isComet()) {
+                request.recycle();
+                request.setFilterChain(null);
+                response.recycle();
+            }
         }
     }
     
+    @Override
+    public boolean asyncDispatch(org.apache.coyote.Request req,
+            org.apache.coyote.Response res, SocketStatus status) throws Exception {
+        Request request = (Request) req.getNote(ADAPTER_NOTES);
+        Response response = (Response) res.getNote(ADAPTER_NOTES);
 
+        if (request == null) {
+            throw new IllegalStateException(
+                    "Dispatch may only happen on an existing request.");
+        }
+        boolean comet = false;
+        boolean success = true;
+        AsyncContextImpl asyncConImpl = (AsyncContextImpl)request.getAsyncContext();
+        try {
+            if (!request.isAsync() && !comet) {
+                // Error or timeout - need to tell listeners the request is over
+                // Have to test this first since state may change while in this
+                // method and this is only required if entering this method in
+                // this state 
+                Context ctxt = (Context) request.getMappingData().context;
+                if (ctxt != null) {
+                    ctxt.fireRequestDestroyEvent(request);
+                }
+                // Lift any suspension (e.g. if sendError() was used by an async
+                // request
+                response.setSuspended(false);
+            }
+
+            if (status==SocketStatus.TIMEOUT) {
+                success = true;
+                if (!asyncConImpl.timeout()) {
+                    asyncConImpl.setErrorState(null);
+                }
+            }
+            if (request.isAsyncDispatching()) {
+                success = true;
+                connector.getService().getContainer().getPipeline().getFirst().invoke(request, response);
+                Throwable t = (Throwable) request.getAttribute(
+                        RequestDispatcher.ERROR_EXCEPTION);
+                if (t != null) {
+                    asyncConImpl.setErrorState(t);
+                }
+            }
+
+            if (request.isComet()) {
+                if (!response.isClosed() && !response.isError()) {
+                    if (request.getAvailable() || (request.getContentLength() > 0 && (!request.isParametersParsed()))) {
+                        // Invoke a read event right away if there are available bytes
+                        if (event(req, res, SocketStatus.OPEN)) {
+                            comet = true;
+                            res.action(ActionCode.COMET_BEGIN, null);
+                        }
+                    } else {
+                        comet = true;
+                        res.action(ActionCode.COMET_BEGIN, null);
+                    }
+                } else {
+                    // Clear the filter chain, as otherwise it will not be reset elsewhere
+                    // since this is a Comet request
+                    request.setFilterChain(null);
+                }
+            }
+            if (!request.isAsync() && !comet) {
+                request.finishRequest();
+                response.finishResponse();
+                req.action(ActionCode.POST_REQUEST , null);
+                ((Context) request.getMappingData().context).logAccess(
+                        request, response,
+                        System.currentTimeMillis() - req.getStartTime(),
+                        false);
+            }
+
+        } catch (IOException e) {
+            success = false;
+            // Ignore
+        } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
+            success = false;
+            log.error(sm.getString("coyoteAdapter.service"), t);
+        } finally {
+            req.getRequestProcessor().setWorkerThreadName(null);
+            // Recycle the wrapper request and response
+            if (!success || (!comet && !request.isAsync())) {
+                request.recycle();
+                response.recycle();
+            } else {
+                // Clear converters so that the minimum amount of memory 
+                // is used by this processor
+                request.clearEncoders();
+                response.clearEncoders();
+            }
+        }
+        return success;
+    }
+    
     /**
      * Service method.
      */
+    @Override
     public void service(org.apache.coyote.Request req, 
-    	                org.apache.coyote.Response res)
+                        org.apache.coyote.Response res)
         throws Exception {
 
         Request request = (Request) req.getNote(ADAPTER_NOTES);
@@ -258,9 +364,9 @@ public class CoyoteAdapter implements Adapter {
         if (request == null) {
 
             // Create objects
-            request = (Request) connector.createRequest();
+            request = connector.createRequest();
             request.setCoyoteRequest(req);
-            response = (Response) connector.createResponse();
+            response = connector.createResponse();
             response.setCoyoteResponse(res);
 
             // Link objects
@@ -282,15 +388,19 @@ public class CoyoteAdapter implements Adapter {
         }
 
         boolean comet = false;
+        boolean async = false;
         
         try {
 
             // Parse and set Catalina and configuration specific 
             // request parameters
             req.getRequestProcessor().setWorkerThreadName(Thread.currentThread().getName());
-            if (postParseRequest(req, request, res, response)) {
+            boolean postParseSuccess = postParseRequest(req, request, res, response);
+            if (postParseSuccess) {
+                //check valves if we support async
+                request.setAsyncSupported(connector.getService().getContainer().getPipeline().isAsyncSupported());
                 // Calling the container
-                connector.getContainer().getPipeline().getFirst().invoke(request, response);
+                connector.getService().getContainer().getPipeline().getFirst().invoke(request, response);
 
                 if (request.isComet()) {
                     if (!response.isClosed() && !response.isError()) {
@@ -298,11 +408,11 @@ public class CoyoteAdapter implements Adapter {
                             // Invoke a read event right away if there are available bytes
                             if (event(req, res, SocketStatus.OPEN)) {
                                 comet = true;
-                                res.action(ActionCode.ACTION_COMET_BEGIN, null);
+                                res.action(ActionCode.COMET_BEGIN, null);
                             }
                         } else {
                             comet = true;
-                            res.action(ActionCode.ACTION_COMET_BEGIN, null);
+                            res.action(ActionCode.COMET_BEGIN, null);
                         }
                     } else {
                         // Clear the filter chain, as otherwise it will not be reset elsewhere
@@ -312,18 +422,32 @@ public class CoyoteAdapter implements Adapter {
                 }
 
             }
-
-            if (!comet) {
+            AsyncContextImpl asyncConImpl = (AsyncContextImpl)request.getAsyncContext();
+            if (asyncConImpl != null) {
+                async = true;
+            } else if (!comet) {
+                request.finishRequest();
                 response.finishResponse();
-                req.action(ActionCode.ACTION_POST_REQUEST , null);
+                if (postParseSuccess) {
+                    // Log only if processing was invoked.
+                    // If postParseRequest() failed, it has already logged it.
+                    ((Context) request.getMappingData().context).logAccess(
+                            request, response,
+                            System.currentTimeMillis() - req.getStartTime(),
+                            false);
+                }
+                req.action(ActionCode.POST_REQUEST , null);
             }
 
         } catch (IOException e) {
-            ;
+            // Ignore
+        } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
+            log.error(sm.getString("coyoteAdapter.service"), t);
         } finally {
             req.getRequestProcessor().setWorkerThreadName(null);
             // Recycle the wrapper request and response
-            if (!comet) {
+            if (!comet && !async) {
                 request.recycle();
                 response.recycle();
             } else {
@@ -337,13 +461,16 @@ public class CoyoteAdapter implements Adapter {
     }
 
 
+    @Override
     public void log(org.apache.coyote.Request req,
             org.apache.coyote.Response res, long time) {
 
         Request request = (Request) req.getNote(ADAPTER_NOTES);
         Response response = (Response) res.getNote(ADAPTER_NOTES);
-
+        boolean create = false;
+        
         if (request == null) {
+            create = true;
             // Create objects
             request = connector.createRequest();
             request.setCoyoteRequest(req);
@@ -364,34 +491,26 @@ public class CoyoteAdapter implements Adapter {
         }
         
         try {
-            // Log at the lowest level available. logAccess() will be
-            // automatically called on parent containers.
-            boolean logged = false;
-            if (request.mappingData != null) {
-                if (request.mappingData.context != null) {
-                    logged = true;
-                    ((Context) request.mappingData.context).logAccess(
-                            request, response, time, true);
-                } else if (request.mappingData.host != null) {
-                    logged = true;
-                    ((Host) request.mappingData.host).logAccess(
-                            request, response, time, true);
-                }
-            }
-            if (!logged) {
-                connector.getService().getContainer().logAccess(
-                        request, response, time, true);
-            }
+            connector.getService().getContainer().logAccess(
+                    request, response, time, true);
         } catch (Throwable t) {
             ExceptionUtils.handleThrowable(t);
             log.warn(sm.getString("coyoteAdapter.accesslogFail"), t);
-        } finally {
+        }
+        
+        if (create) {
             request.recycle();
             response.recycle();
         }
     }
     
     
+    @Override
+    public String getDomain() {
+        return connector.getDomain();
+    }
+
+
     // ------------------------------------------------------ Protected Methods
 
 
@@ -400,12 +519,10 @@ public class CoyoteAdapter implements Adapter {
      */
     protected boolean postParseRequest(org.apache.coyote.Request req, 
                                        Request request,
-    		                       org.apache.coyote.Response res, 
+                                       org.apache.coyote.Response res, 
                                        Response response)
             throws Exception {
 
-        // XXX the processor needs to set a correct scheme and port prior to this point, 
-        // in ajp13 protocols dont make sense to get the port from the connector..
         // XXX the processor may have set a correct scheme and port prior to this point, 
         // in ajp13 protocols dont make sense to get the port from the connector...
         // otherwise, use connector configuration
@@ -433,15 +550,15 @@ public class CoyoteAdapter implements Adapter {
             req.serverName().setString(proxyName);
         }
 
-        // Copy the raw URI to the decoded URI
+        // Copy the raw URI to the decodedURI
         MessageBytes decodedURI = req.decodedURI();
         decodedURI.duplicate(req.requestURI());
-
+        
         // Parse the path parameters. This will:
         //   - strip out the path parameters
         //   - convert the decodedURI to bytes
         parsePathParameters(req, request);
-
+        
         // URI decoding
         // %xx decoding of the URL
         try {
@@ -490,31 +607,127 @@ public class CoyoteAdapter implements Adapter {
             serverName = req.localName();
             if (serverName.isNull()) {
                 // well, they did ask for it
-                res.action(ActionCode.ACTION_REQ_LOCAL_NAME_ATTRIBUTE, null);
+                res.action(ActionCode.REQ_LOCAL_NAME_ATTRIBUTE, null);
             }
         } else {
             serverName = req.serverName();
         }
-        connector.getMapper().map(serverName, decodedURI, 
-                                  request.getMappingData());
-        request.setContext((Context) request.getMappingData().context);
-        
-        // If there is no context at this point, it is likely no ROOT context
-        // has been deployed
-        if (request.getContext() == null) {
-            res.setStatus(404);
-            res.setMessage("Not found");
-            connector.getService().getContainer().logAccess(request, response, 0, true);
-            return false;
+        if (request.isAsyncStarted()) {
+            //TODO SERVLET3 - async
+            //reset mapping data, should prolly be done elsewhere
+            request.getMappingData().recycle();
         }
         
-        // Had to do this after the context was set.
-        // Unfortunately parseSessionId is still necessary as it 
-        // affects the final URL. Safe as session cookies still 
-        // haven't been parsed.
-        if (isURLRewritingDisabled(request))
-            clearRequestedSessionURL(request);
-        request.setWrapper((Wrapper) request.getMappingData().wrapper);
+        boolean mapRequired = true;
+        String version = null;
+        
+        while (mapRequired) {
+            if (version != null) {
+                // Once we have a version - that is it
+                mapRequired = false;
+            }
+            // This will map the the latest version by default
+            connector.getMapper().map(serverName, decodedURI, version,
+                                      request.getMappingData());
+            request.setContext((Context) request.getMappingData().context);
+            request.setWrapper((Wrapper) request.getMappingData().wrapper);
+
+            // Single contextVersion therefore no possibility of remap
+            if (request.getMappingData().contexts == null) {
+                mapRequired = false;
+            }
+
+            // If there is no context at this point, it is likely no ROOT context
+            // has been deployed
+            if (request.getContext() == null) {
+                res.setStatus(404);
+                res.setMessage("Not found");
+                // No context, so use host
+                Host host = request.getHost();
+                // Make sure there is a host (might not be during shutdown)
+                if (host != null) {
+                    host.logAccess(request, response, 0, true);
+                }
+                return false;
+            }
+        
+            // Now we have the context, we can parse the session ID from the URL
+            // (if any). Need to do this before we redirect in case we need to
+            // include the session id in the redirect
+            String sessionID = null;
+            if (request.getServletContext().getEffectiveSessionTrackingModes()
+                    .contains(SessionTrackingMode.URL)) {
+                
+                // Get the session ID if there was one
+                sessionID = request.getPathParameter(
+                        ApplicationSessionCookieConfig.getSessionUriParamName(
+                                request.getContext()));
+                if (sessionID != null) {
+                    request.setRequestedSessionId(sessionID);
+                    request.setRequestedSessionURL(true);
+                }
+            }
+
+            // Look for session ID in cookies and SSL session
+            parseSessionCookiesId(req, request);
+            parseSessionSslId(request);
+            
+            sessionID = request.getRequestedSessionId();
+            
+            if (mapRequired) {
+                if (sessionID == null) {
+                    // No session means no possibility of needing to remap
+                    mapRequired = false;
+                } else {
+                    // Find the context associated with the session
+                    Object[] objs = request.getMappingData().contexts;
+                    for (int i = (objs.length); i > 0; i--) {
+                        Context ctxt = (Context) objs[i - 1];
+                        if (ctxt.getManager().findSession(sessionID) != null) {
+                            // Was the correct context already mapped?
+                            if (ctxt.equals(request.getMappingData().context)) {
+                                mapRequired = false;
+                            } else {
+                                // Set version so second time through mapping the
+                                // correct context is found
+                                version = ctxt.getWebappVersion();
+                                // Reset mapping 
+                                request.getMappingData().recycle();
+                                break;
+                            }
+                        }
+                    }
+                    if (version == null) {
+                        // No matching context found. No need to re-map
+                        mapRequired = false;
+                    }
+                }                
+            }
+
+        }
+
+        // Possible redirect
+        MessageBytes redirectPathMB = request.getMappingData().redirectPath;
+        if (!redirectPathMB.isNull()) {
+            String redirectPath = urlEncoder.encode(redirectPathMB.toString());
+            String query = request.getQueryString();
+            if (request.isRequestedSessionIdFromURL()) {
+                // This is not optimal, but as this is not very common, it
+                // shouldn't matter
+                redirectPath = redirectPath + ";" +
+                    ApplicationSessionCookieConfig.getSessionUriParamName(
+                            request.getContext()) +
+                    "=" + request.getRequestedSessionId();
+            }
+            if (query != null) {
+                // This is not optimal, but as this is not very common, it
+                // shouldn't matter
+                redirectPath = redirectPath + "?" + query;
+            }
+            response.sendRedirect(redirectPath);
+            request.getContext().logAccess(request, response, 0, true);
+            return false;
+        }
 
         // Filter trace method
         if (!connector.getAllowTrace() 
@@ -543,47 +756,9 @@ public class CoyoteAdapter implements Adapter {
             return false;
         }
 
-        // Possible redirect
-        MessageBytes redirectPathMB = request.getMappingData().redirectPath;
-        if (!redirectPathMB.isNull()) {
-            String redirectPath = urlEncoder.encode(redirectPathMB.toString());
-            String query = request.getQueryString();
-            if (request.isRequestedSessionIdFromURL()) {
-                // This is not optimal, but as this is not very common, it
-                // shouldn't matter
-                redirectPath = redirectPath + ";" + Globals.SESSION_PARAMETER_NAME + "=" 
-                    + request.getRequestedSessionId();
-            }
-            if (query != null) {
-                // This is not optimal, but as this is not very common, it
-                // shouldn't matter
-                redirectPath = redirectPath + "?" + query;
-            }
-            response.sendRedirect(redirectPath);
-            request.getContext().logAccess(request, response, 0, true);
-            return false;
-        }
-
-        // Parse session Id
-        String sessionID =
-            request.getPathParameter(Globals.SESSION_PARAMETER_NAME);
-        if (sessionID != null) {
-            request.setRequestedSessionId(sessionID);
-            request.setRequestedSessionURL(true);
-        }
-        parseSessionCookiesId(req, request);
-
         return true;
     }
 
-    private boolean isURLRewritingDisabled(Request request) {
-        Context context = (Context) request.getMappingData().context;
-        if (context != null)
-            return (context.isDisableURLRewriting());
-        else
-            return (false);
-    }
-
 
     /**
      * Extract the path parameters from the request. This assumes parameters are
@@ -693,35 +868,27 @@ public class CoyoteAdapter implements Adapter {
             semicolon = uriBC.indexOf(';', semicolon);
         }
     }
-
-
+    
+    
     /**
-     * Parse session id in URL.
-     * @deprecated Not used since 6.0.30
+     * Look for SSL session ID if required. Only look for SSL Session ID if it
+     * is the only tracking method enabled.
      */
-    @Deprecated
-    protected void parseSessionId(org.apache.coyote.Request req, Request request) {
-
-        parsePathParameters(req, request);
-
-        String sessionID =
-            request.getPathParameter(Globals.SESSION_PARAMETER_NAME);
-        if (sessionID != null) {
-            request.setRequestedSessionId(sessionID);
-            request.setRequestedSessionURL(true);
-        } else {
-            clearRequestedSessionURL(request);
+    protected void parseSessionSslId(Request request) {
+        if (request.getRequestedSessionId() == null &&
+                SSL_ONLY.equals(request.getServletContext()
+                        .getEffectiveSessionTrackingModes()) &&
+                        request.connector.secure) {
+            // TODO Is there a better way to map SSL sessions to our sesison ID?
+            // TODO The request.getAttribute() will cause a number of other SSL
+            //      attribute to be populated. Is this a performance concern?
+            request.setRequestedSessionId(
+                    request.getAttribute(SSLSupport.SESSION_ID_KEY).toString());
+            request.setRequestedSessionSSL(true);
         }
-
     }
-
-
-    private void clearRequestedSessionURL(Request request) {
-        request.setRequestedSessionId(null);
-        request.setRequestedSessionURL(false);
-    }
-
-
+    
+    
     /**
      * Parse session id in URL.
      */
@@ -732,17 +899,20 @@ public class CoyoteAdapter implements Adapter {
         // from a parent context with a session ID may be present which would
         // overwrite the valid session ID encoded in the URL
         Context context = (Context) request.getMappingData().context;
-        if (context != null && !context.getCookies())
+        if (context != null && !context.getServletContext()
+                .getEffectiveSessionTrackingModes().contains(
+                        SessionTrackingMode.COOKIE))
             return;
-
+        
         // Parse session id from cookies
         Cookies serverCookies = req.getCookies();
         int count = serverCookies.getCookieCount();
         if (count <= 0)
             return;
 
-        String sessionCookieName = getSessionCookieName(context);
-        
+        String sessionCookieName =
+            ApplicationSessionCookieConfig.getSessionCookieName(context);
+
         for (int i = 0; i < count; i++) {
             ServerCookie scookie = serverCookies.getCookie(i);
             if (scookie.getName().equals(sessionCookieName)) {
@@ -797,7 +967,7 @@ public class CoyoteAdapter implements Adapter {
             }
             if (conv != null) {
                 try {
-                    conv.convert(bc, cc);
+                    conv.convert(bc, cc, cc.getBuffer().length - cc.getEnd());
                     uri.setChars(cc.getBuffer(), cc.getStart(), 
                                  cc.getLength());
                     return;
@@ -1019,6 +1189,9 @@ public class CoyoteAdapter implements Adapter {
     }
 
 
+    // ------------------------------------------------------ Protected Methods
+
+
     /**
      * Copy an array of bytes to a different position. Used during 
      * normalization.
@@ -1028,18 +1201,4 @@ public class CoyoteAdapter implements Adapter {
             b[pos + dest] = b[pos + src];
         }
     }
-    private String getSessionCookieName(Context context) {
-        
-        String result = null;
-        
-        if (context != null) {
-            result = context.getSessionCookieName();
-        }
-        
-        if (result == null) {
-            result = Globals.SESSION_COOKIE_NAME;
-        }
-        
-        return result;
-    }
 }
