@@ -25,10 +25,13 @@ import org.apache.catalina.ha.ClusterMessageBase;
  * @author Filip Hanik
  * @author Peter Rossbach
  * 
- * @version $Id: SessionMessageImpl.java 956838 2010-06-22 10:09:14Z kfujino $
+ * @version $Id: SessionMessageImpl.java 1026793 2010-10-24 13:25:36Z markt $
  */
-public class SessionMessageImpl extends ClusterMessageBase implements SessionMessage, java.io.Serializable {
+public class SessionMessageImpl extends ClusterMessageBase implements SessionMessage {
     
+    private static final long serialVersionUID = 1L;
+
+
     public SessionMessageImpl() {
     }
     
@@ -100,21 +103,25 @@ public class SessionMessageImpl extends ClusterMessageBase implements SessionMes
      * returns the event type
      * @return one of the event types EVT_XXXX
      */
+    @Override
     public int getEventType() { return mEvtType; }
 
     /**
      * @return the serialized data for the session
      */
+    @Override
     public byte[] getSession() { return mSession;}
 
     /**
      * @return the session ID for the session
      */
+    @Override
     public String getSessionID(){ return mSessionID; }
     
     /**
      * set message send time but only the first setting works (one shot)
      */
+    @Override
     public void setTimestamp(long time) {
         synchronized(this) {
             if(!timestampSet) {
@@ -124,12 +131,14 @@ public class SessionMessageImpl extends ClusterMessageBase implements SessionMes
         }
     }
     
+    @Override
     public long getTimestamp() { return serializationTimestamp;}
     
     /**
      * clear text event type name (for logging purpose only) 
-     * @return the event type in a string representating, useful for debugging
+     * @return the event type in a string representation, useful for debugging
      */
+    @Override
     public String getEventTypeString()
     {
         switch (mEvtType)
@@ -146,16 +155,20 @@ public class SessionMessageImpl extends ClusterMessageBase implements SessionMes
         }
     }
 
+    @Override
     public String getContextName() {
        return mContextName;
     }
+    @Override
     public String getUniqueId() {
         return uniqueId;
     }
+    @Override
     public void setUniqueId(String uniqueId) {
         this.uniqueId = uniqueId;
     }
     
+    @Override
     public String toString() {
         return getEventTypeString() + "#" + getContextName() + "#" + getSessionID() ;
     }
