@@ -29,32 +29,35 @@ import java.security.Principal;
 import java.security.cert.X509Certificate;
 import java.util.ArrayList;
 
-import javax.management.Attribute;
-import javax.management.MBeanRegistration;
-import javax.management.MBeanServer;
-import javax.management.ObjectName;
 import javax.servlet.http.HttpServletResponse;
 
 import org.apache.catalina.Container;
 import org.apache.catalina.Context;
-import org.apache.catalina.Globals;
-import org.apache.catalina.Lifecycle;
+import org.apache.catalina.Engine;
+import org.apache.catalina.Host;
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.LifecycleListener;
+import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Realm;
+import org.apache.catalina.Server;
+import org.apache.catalina.Service;
+import org.apache.catalina.Wrapper;
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.connector.Response;
-import org.apache.catalina.core.ContainerBase;
+import org.apache.catalina.core.ApplicationSessionCookieConfig;
 import org.apache.catalina.deploy.LoginConfig;
-import org.apache.catalina.deploy.SecurityConstraint;
 import org.apache.catalina.deploy.SecurityCollection;
-import org.apache.catalina.util.HexUtils;
-import org.apache.catalina.util.LifecycleSupport;
+import org.apache.catalina.deploy.SecurityConstraint;
+import org.apache.catalina.mbeans.MBeanUtils;
+import org.apache.catalina.util.LifecycleMBeanBase;
 import org.apache.catalina.util.MD5Encoder;
-import org.apache.catalina.util.StringManager;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
-import org.apache.tomcat.util.modeler.Registry;
+import org.apache.tomcat.util.buf.HexUtils;
+import org.apache.tomcat.util.res.StringManager;
+import org.ietf.jgss.GSSContext;
+import org.ietf.jgss.GSSCredential;
+import org.ietf.jgss.GSSException;
+import org.ietf.jgss.GSSName;
 
 /**
  * Simple implementation of <b>Realm</b> that reads an XML file to configure
@@ -62,13 +65,12 @@ import org.apache.tomcat.util.modeler.Registry;
  * location) are identical to those currently supported by Tomcat 3.X.
  *
  * @author Craig R. McClanahan
- * @version $Id: RealmBase.java 1158180 2011-08-16 10:11:43Z markt $
+ * @version $Id: RealmBase.java 1132702 2011-06-06 16:53:58Z markt $
  */
 
-public abstract class RealmBase
-    implements Lifecycle, Realm, MBeanRegistration {
+public abstract class RealmBase extends LifecycleMBeanBase implements Realm {
 
-    private static Log log = LogFactory.getLog(RealmBase.class);
+    private static final Log log = LogFactory.getLog(RealmBase.class);
 
     // ----------------------------------------------------- Instance Variables
 
@@ -106,16 +108,10 @@ public abstract class RealmBase
         "org.apache.catalina.realm.RealmBase/1.0";
 
 
-    /**
-     * The lifecycle event support for this component.
-     */
-    protected LifecycleSupport lifecycle = new LifecycleSupport(this);
-
-
     /**
      * The MessageDigest object for digesting user credentials (passwords).
      */
-    protected MessageDigest md = null;
+    protected volatile MessageDigest md = null;
 
 
     /**
@@ -127,22 +123,16 @@ public abstract class RealmBase
     /**
      * MD5 message digest provider.
      */
-    protected static MessageDigest md5Helper;
+    protected static volatile MessageDigest md5Helper;
 
 
     /**
      * The string manager for this package.
      */
-    protected static StringManager sm =
+    protected static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
 
-    /**
-     * Has this component been started?
-     */
-    protected boolean started = false;
-
-
     /**
      * The property change support for this component.
      */
@@ -161,12 +151,20 @@ public abstract class RealmBase
     protected AllRolesMode allRolesMode = AllRolesMode.STRICT_MODE;
     
 
+    /**
+     * When processing users authenticated via the GSS-API, should any
+     * &quot;@...&quot; be stripped from the end of the user name?
+     */
+    protected boolean stripRealmForGss = true;
+
+    
     // ------------------------------------------------------------- Properties
 
 
     /**
      * Return the Container with which this Realm has been associated.
      */
+    @Override
     public Container getContainer() {
 
         return (container);
@@ -179,6 +177,7 @@ public abstract class RealmBase
      *
      * @param container The associated Container
      */
+    @Override
     public void setContainer(Container container) {
 
         Container oldContainer = this.container;
@@ -250,6 +249,7 @@ public abstract class RealmBase
      * the corresponding version number, in the format
      * <code>&lt;description&gt;/&lt;version&gt;</code>.
      */
+    @Override
     public String getInfo() {
 
         return info;
@@ -279,15 +279,25 @@ public abstract class RealmBase
     }
 
 
+    public boolean isStripRealmForGss() {
+        return stripRealmForGss;
+    }
+
+
+    public void setStripRealmForGss(boolean stripRealmForGss) {
+        this.stripRealmForGss = stripRealmForGss;
+    }
+
+
     // --------------------------------------------------------- Public Methods
 
 
-    
     /**
      * Add a property change listener to this component.
      *
      * @param listener The listener to add
      */
+    @Override
     public void addPropertyChangeListener(PropertyChangeListener listener) {
 
         support.addPropertyChangeListener(listener);
@@ -303,6 +313,7 @@ public abstract class RealmBase
      * @param credentials Password or other credentials to use in
      *  authenticating this username
      */
+    @Override
     public Principal authenticate(String username, String credentials) {
 
         String serverCredentials = getPassword(username);
@@ -331,21 +342,6 @@ public abstract class RealmBase
     }
 
 
-    /**
-     * Return the Principal associated with the specified username and
-     * credentials, if there is one; otherwise return <code>null</code>.
-     *
-     * @param username Username of the Principal to look up
-     * @param credentials Password or other credentials to use in
-     *  authenticating this username
-     */
-    public Principal authenticate(String username, byte[] credentials) {
-
-        return (authenticate(username, credentials.toString()));
-
-    }
-
-
     /**
      * Return the Principal associated with the specified username, which
      * matches the digest calculated using the given parameters using the
@@ -353,14 +349,15 @@ public abstract class RealmBase
      *
      * @param username Username of the Principal to look up
      * @param clientDigest Digest which has been submitted by the client
-     * @param nonce Unique (or supposedly unique) token which has been used
+     * @param nOnce Unique (or supposedly unique) token which has been used
      * for this request
      * @param realm Realm name
      * @param md5a2 Second MD5 digest used to calculate the digest :
      * MD5(Method + ":" + uri)
      */
+    @Override
     public Principal authenticate(String username, String clientDigest,
-                                  String nonce, String nc, String cnonce,
+                                  String nOnce, String nc, String cnonce,
                                   String qop, String realm,
                                   String md5a2) {
 
@@ -369,9 +366,9 @@ public abstract class RealmBase
             return null;
         String serverDigestValue;
         if (qop == null) {
-            serverDigestValue = md5a1 + ":" + nonce + ":" + md5a2;
+            serverDigestValue = md5a1 + ":" + nOnce + ":" + md5a2;
         } else {
-            serverDigestValue = md5a1 + ":" + nonce + ":" + nc + ":" +
+            serverDigestValue = md5a1 + ":" + nOnce + ":" + nc + ":" +
                     cnonce + ":" + qop + ":" + md5a2;
         }
 
@@ -395,18 +392,18 @@ public abstract class RealmBase
 
         if (log.isDebugEnabled()) {
             log.debug("Digest : " + clientDigest + " Username:" + username 
-                    + " ClientSigest:" + clientDigest + " nonce:" + nonce 
+                    + " ClientSigest:" + clientDigest + " nOnce:" + nOnce 
                     + " nc:" + nc + " cnonce:" + cnonce + " qop:" + qop 
                     + " realm:" + realm + "md5a2:" + md5a2 
                     + " Server digest:" + serverDigest);
         }
         
-        if (serverDigest.equals(clientDigest))
+        if (serverDigest.equals(clientDigest)) {
             return getPrincipal(username);
-        else
-            return null;
-    }
+        }
 
+        return null;
+    }
 
 
     /**
@@ -416,6 +413,7 @@ public abstract class RealmBase
      * @param certs Array of client certificates, with the first one in
      *  the array being the certificate of the client itself.
      */
+    @Override
     public Principal authenticate(X509Certificate certs[]) {
 
         if ((certs == null) || (certs.length < 1))
@@ -445,12 +443,58 @@ public abstract class RealmBase
     }
 
     
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public Principal authenticate(GSSContext gssContext, boolean storeCred) {
+        if (gssContext.isEstablished()) {
+            GSSName gssName = null;
+            try {
+                gssName = gssContext.getSrcName();
+            } catch (GSSException e) {
+                log.warn(sm.getString("realmBase.gssNameFail"), e);
+            }
+            
+            if (gssName!= null) {
+                String name = gssName.toString();
+                
+                if (isStripRealmForGss()) {
+                    int i = name.indexOf('@');
+                    if (i > 0) {
+                        // Zero so we don;t leave a zero length name
+                        name = name.substring(0, i);
+                    }
+                }
+                GSSCredential gssCredential = null;
+                if (storeCred && gssContext.getCredDelegState()) {
+                    try {
+                        gssCredential = gssContext.getDelegCred();
+                    } catch (GSSException e) {
+                        if (log.isDebugEnabled()) {
+                            log.debug(sm.getString(
+                                    "realmBase.delegatedCredentialFail", name),
+                                    e);
+                        }
+                    }
+                }
+                return getPrincipal(name, gssCredential);
+            }
+        }
+        
+        // Fail in all other cases
+        return null;
+    }
+
+    
     /**
      * Execute a periodic task, such as reloading, etc. This method will be
      * invoked inside the classloading context of this container. Unexpected
      * throwables will be caught and logged.
      */
+    @Override
     public void backgroundProcess() {
+        // NOOP in base class
     }
 
 
@@ -461,6 +505,7 @@ public abstract class RealmBase
      * @param request Request we are processing
      * @param context Context the Request is mapped to
      */
+    @Override
     public SecurityConstraint [] findSecurityConstraints(Request request,
                                                          Context context) {
 
@@ -490,14 +535,14 @@ public abstract class RealmBase
             // If collection is null, continue to avoid an NPE
             // See Bugzilla 30624
             if ( collection == null) {
-		continue;
+                continue;
             }
 
             if (log.isDebugEnabled()) {
                 log.debug("  Checking constraint '" + constraints[i] +
                     "' against " + method + " " + uri + " --> " +
                     constraints[i].included(uri, method));
-	    }
+            }
 
             for(int j=0; j < collection.length; j++){
                 String [] patterns = collection[j].findPatterns();
@@ -505,7 +550,7 @@ public abstract class RealmBase
                 // If patterns is null, continue to avoid an NPE
                 // See Bugzilla 30624
                 if ( patterns == null) {
-		    continue;
+                    continue;
                 }
 
                 for(int k=0; k < patterns.length; k++) {
@@ -534,14 +579,14 @@ public abstract class RealmBase
             // If collection is null, continue to avoid an NPE
             // See Bugzilla 30624
             if ( collection == null) {
-		continue;
+                continue;
             }
 
             if (log.isDebugEnabled()) {
                 log.debug("  Checking constraint '" + constraints[i] +
                     "' against " + method + " " + uri + " --> " +
                     constraints[i].included(uri, method));
-	    }
+            }
 
             for(int j=0; j < collection.length; j++){
                 String [] patterns = collection[j].findPatterns();
@@ -549,7 +594,7 @@ public abstract class RealmBase
                 // If patterns is null, continue to avoid an NPE
                 // See Bugzilla 30624
                 if ( patterns == null) {
-		    continue;
+                    continue;
                 }
 
                 boolean matched = false;
@@ -600,14 +645,14 @@ public abstract class RealmBase
             // If collection is null, continue to avoid an NPE
             // See Bugzilla 30624
             if ( collection == null) {
-		continue;
+                continue;
             }
             
             if (log.isDebugEnabled()) {
                 log.debug("  Checking constraint '" + constraints[i] +
                     "' against " + method + " " + uri + " --> " +
                     constraints[i].included(uri, method));
-	    }
+            }
 
             boolean matched = false;
             int pos = -1;
@@ -617,7 +662,7 @@ public abstract class RealmBase
                 // If patterns is null, continue to avoid an NPE
                 // See Bugzilla 30624
                 if ( patterns == null) {
-		    continue;
+                    continue;
                 }
 
                 for(int k=0; k < patterns.length && !matched; k++) {
@@ -657,14 +702,14 @@ public abstract class RealmBase
             // If collection is null, continue to avoid an NPE
             // See Bugzilla 30624
             if ( collection == null) {
-		continue;
+                continue;
             }
 
             if (log.isDebugEnabled()) {
                 log.debug("  Checking constraint '" + constraints[i] +
                     "' against " + method + " " + uri + " --> " +
                     constraints[i].included(uri, method));
-	    }
+            }
 
             for(int j=0; j < collection.length; j++){
                 String [] patterns = collection[j].findPatterns();
@@ -672,7 +717,7 @@ public abstract class RealmBase
                 // If patterns is null, continue to avoid an NPE
                 // See Bugzilla 30624
                 if ( patterns == null) {
-		    continue;
+                    continue;
                 }
 
                 boolean matched = false;
@@ -725,6 +770,7 @@ public abstract class RealmBase
      *
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public boolean hasResourcePermission(Request request,
                                          Response response,
                                          SecurityConstraint []constraints,
@@ -783,21 +829,21 @@ public abstract class RealmBase
             if (roles.length == 0 && !constraint.getAllRoles()) {
                 if(constraint.getAuthConstraint()) {
                     if( log.isDebugEnabled() )
-                        log.debug("No roles ");
+                        log.debug("No roles");
                     status = false; // No listed roles means no access at all
                     denyfromall = true;
                     break;
-                } else {
-                    if(log.isDebugEnabled())
-                        log.debug("Passing all access");
-                    status = true;
                 }
+                
+                if(log.isDebugEnabled())
+                    log.debug("Passing all access");
+                status = true;
             } else if (principal == null) {
                 if (log.isDebugEnabled())
                     log.debug("  No user authenticated, cannot grant access");
             } else {
                 for (int j = 0; j < roles.length; j++) {
-                    if (hasRole(principal, roles[j])) {
+                    if (hasRole(null, principal, roles[j])) {
                         status = true;
                         if( log.isDebugEnabled() )
                             log.debug( "Role found:  " + roles[j]);
@@ -862,18 +908,21 @@ public abstract class RealmBase
      * @param principal Principal for whom the role is to be checked
      * @param role Security role to be checked
      */
-    public boolean hasRole(Principal principal, String role) {
+    @Override
+    public boolean hasRole(Wrapper wrapper, Principal principal, String role) {
+        // Check for a role alias defined in a <security-role-ref> element
+        if (wrapper != null) {
+            String realRole = wrapper.findSecurityReference(role);
+            if (realRole != null)
+                role = realRole;
+        }
 
-        // Should be overriten in JAASRealm - to avoid pretty inefficient conversions
+        // Should be overridden in JAASRealm - to avoid pretty inefficient conversions
         if ((principal == null) || (role == null) ||
             !(principal instanceof GenericPrincipal))
             return (false);
 
         GenericPrincipal gp = (GenericPrincipal) principal;
-        if (!(gp.getRealm() == this)) {
-            if(log.isDebugEnabled())
-                log.debug("Different realm " + this + " " + gp.getRealm());//    return (false);
-        }
         boolean result = gp.hasRole(role);
         if (log.isDebugEnabled()) {
             String name = principal.getName();
@@ -899,6 +948,7 @@ public abstract class RealmBase
      *
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public boolean hasUserDataPermission(Request request,
                                          Response response,
                                          SecurityConstraint []constraints)
@@ -945,7 +995,7 @@ public abstract class RealmBase
         }
 
         // Redirect to the corresponding SSL port
-        StringBuffer file = new StringBuffer();
+        StringBuilder file = new StringBuilder();
         String protocol = "https";
         String host = request.getServerName();
         // Protocol
@@ -960,7 +1010,8 @@ public abstract class RealmBase
         if ((requestedSessionId != null) &&
             request.isRequestedSessionIdFromURL()) {
             file.append(";");
-            file.append(Globals.SESSION_PARAMETER_NAME);
+            file.append(ApplicationSessionCookieConfig.getSessionUriParamName(
+                    request.getContext()));
             file.append("=");
             file.append(requestedSessionId);
         }
@@ -982,6 +1033,7 @@ public abstract class RealmBase
      *
      * @param listener The listener to remove
      */
+    @Override
     public void removePropertyChangeListener(PropertyChangeListener listener) {
 
         support.removePropertyChangeListener(listener);
@@ -989,65 +1041,27 @@ public abstract class RealmBase
     }
 
 
-    // ------------------------------------------------------ Lifecycle Methods
+    @Override
+    protected void initInternal() throws LifecycleException {
 
+        super.initInternal();
 
-    /**
-     * Add a lifecycle event listener to this component.
-     *
-     * @param listener The listener to add
-     */
-    public void addLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.addLifecycleListener(listener);
-
+        // We want logger as soon as possible
+        if (container != null) {
+            this.containerLog = container.getLogger();
+        }
     }
-
-
-    /**
-     * Get the lifecycle listeners associated with this lifecycle. If this 
-     * Lifecycle has no listeners registered, a zero-length array is returned.
-     */
-    public LifecycleListener[] findLifecycleListeners() {
-
-        return lifecycle.findLifecycleListeners();
-
-    }
-
-
-    /**
-     * Remove a lifecycle event listener from this component.
-     *
-     * @param listener The listener to remove
-     */
-    public void removeLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.removeLifecycleListener(listener);
-
-    }
-
+        
     /**
      * Prepare for the beginning of active use of the public methods of this
-     * component.  This method should be called before any of the public
-     * methods of this component are utilized.  It should also send a
-     * LifecycleEvent of type START_EVENT to any registered listeners.
+     * component and implement the requirements of
+     * {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
      *  that prevents this component from being used
      */
-    public void start() throws LifecycleException {
-
-        // Validate and update our current component state
-        if (started) {
-            if(log.isInfoEnabled())
-                log.info(sm.getString("realmBase.alreadyStarted"));
-            return;
-        }
-        if( !initialized ) {
-            init();
-        }
-        lifecycle.fireLifecycleEvent(START_EVENT, null);
-        started = true;
+    @Override
+    protected void startInternal() throws LifecycleException {
 
         // Create a MessageDigest instance for credentials, if desired
         if (digest != null) {
@@ -1059,52 +1073,40 @@ public abstract class RealmBase
             }
         }
 
+        setState(LifecycleState.STARTING);
     }
 
 
     /**
      * Gracefully terminate the active use of the public methods of this
-     * component.  This method should be the last one called on a given
-     * instance of this component.  It should also send a LifecycleEvent
-     * of type STOP_EVENT to any registered listeners.
+     * component and implement the requirements of
+     * {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
      *  that needs to be reported
      */
-    public void stop()
-        throws LifecycleException {
-
-        // Validate and update our current component state
-        if (!started) {
-            if(log.isInfoEnabled())
-                log.info(sm.getString("realmBase.notStarted"));
-            return;
-        }
-        lifecycle.fireLifecycleEvent(STOP_EVENT, null);
-        started = false;
+    @Override
+    protected void stopInternal() throws LifecycleException {
 
+        setState(LifecycleState.STOPPING);
+        
         // Clean up allocated resources
         md = null;
-        
-        destroy();
-    
     }
     
-    public void destroy() {
     
-        // unregister this realm
-        if ( oname!=null ) {   
-            try {   
-                Registry.getRegistry(null, null).unregisterComponent(oname); 
-                if(log.isDebugEnabled())
-                    log.debug( "unregistering realm " + oname );   
-            } catch( Exception ex ) {   
-                log.error( "Can't unregister realm " + oname, ex);   
-            }      
-        }
-          
+    /**
+     * Return a String representation of this component.
+     */
+    @Override
+    public String toString() {
+        StringBuilder sb = new StringBuilder("Realm[");
+        sb.append(getName());
+        sb.append(']');
+        return sb.toString();
     }
-
+    
+    
     // ------------------------------------------------------ Protected Methods
 
 
@@ -1140,7 +1142,7 @@ public abstract class RealmBase
                 }
                 md.update(bytes);
 
-                return (HexUtils.convert(md.digest()));
+                return (HexUtils.toHexString(md.digest()));
             } catch (Exception e) {
                 log.error(sm.getString("realmBase.digest"), e);
                 return (credentials);
@@ -1166,11 +1168,11 @@ public abstract class RealmBase
             }
         }
 
-    	if (hasMessageDigest()) {
-    		// Use pre-generated digest
-    		return getPassword(username);
-    	}
-    	
+        if (hasMessageDigest()) {
+            // Use pre-generated digest
+            return getPassword(username);
+        }
+            
         String digestValue = username + ":" + realmName + ":"
             + getPassword(username);
 
@@ -1223,6 +1225,41 @@ public abstract class RealmBase
     protected abstract Principal getPrincipal(String username);
 
 
+    protected Principal getPrincipal(String username,
+            GSSCredential gssCredential) {
+        Principal p = getPrincipal(username);
+        
+        if (p instanceof GenericPrincipal) {
+            ((GenericPrincipal) p).setGssCredential(gssCredential);
+        }
+        
+        return p;
+    }
+
+    /**
+     * Return the Server object that is the ultimate parent for the container
+     * with which this Realm is associated. If the server cannot be found (eg
+     * because the container hierarchy is not complete), <code>null</code> is
+     * returned.
+     */
+    protected Server getServer() {
+        Container c = container;
+        if (c instanceof Context) {
+            c = c.getParent();
+        }
+        if (c instanceof Host) {
+            c = c.getParent();
+        }
+        if (c instanceof Engine) {
+            Service s = ((Engine)c).getService();
+            if (s != null) {
+                return s.getServer();
+            }
+        }
+        return null;
+    }
+
+    
     // --------------------------------------------------------- Static Methods
 
 
@@ -1236,7 +1273,7 @@ public abstract class RealmBase
      * @param algorithm Algorithm used to do the digest
      * @param encoding Character encoding of the string to digest
      */
-    public final static String Digest(String credentials, String algorithm,
+    public static final String Digest(String credentials, String algorithm,
                                       String encoding) {
 
         try {
@@ -1253,7 +1290,7 @@ public abstract class RealmBase
             }
 
             // Digest the credentials and return as hexadecimal
-            return (HexUtils.convert(md.digest()));
+            return (HexUtils.toHexString(md.digest()));
         } catch(Exception ex) {
             log.error(ex);
             return credentials;
@@ -1291,34 +1328,23 @@ public abstract class RealmBase
 
 
     // -------------------- JMX and Registration  --------------------
-    protected String type;
-    protected String domain;
-    protected String host;
-    protected String path;
+
+    @Override
+    public String getObjectNameKeyProperties() {
+        
+        StringBuilder keyProperties = new StringBuilder("type=Realm");
+        keyProperties.append(getRealmSuffix());
+        keyProperties.append(MBeanUtils.getContainerKeyProperties(container));
+
+        return keyProperties.toString();
+    }
+
+    @Override
+    public String getDomainInternal() {
+        return MBeanUtils.getDomain(container);
+    }
+
     protected String realmPath = "/realm0";
-    protected ObjectName oname;
-    protected ObjectName controller;
-    protected MBeanServer mserver;
-
-    public ObjectName getController() {
-        return controller;
-    }
-
-    public void setController(ObjectName controller) {
-        this.controller = controller;
-    }
-
-    public ObjectName getObjectName() {
-        return oname;
-    }
-
-    public String getDomain() {
-        return domain;
-    }
-
-    public String getType() {
-        return type;
-    }
 
     public String getRealmPath() {
         return realmPath;
@@ -1328,80 +1354,6 @@ public abstract class RealmBase
         realmPath = theRealmPath;
     }
 
-    public ObjectName preRegister(MBeanServer server,
-                                  ObjectName name) throws Exception {
-        oname=name;
-        mserver=server;
-        domain=name.getDomain();
-
-        type=name.getKeyProperty("type");
-        host=name.getKeyProperty("host");
-        path=name.getKeyProperty("path");
-
-        return name;
-    }
-
-    public void postRegister(Boolean registrationDone) {
-    }
-
-    public void preDeregister() throws Exception {
-    }
-
-    public void postDeregister() {
-    }
-
-    protected boolean initialized=false;
-    
-    public void init() {
-        if( initialized && container != null ) return;
-
-        // We want logger as soon as possible
-        if (container != null) {
-            this.containerLog = container.getLogger();
-        }
-        
-        initialized=true;
-        if( container== null ) {
-            ObjectName parent=null;
-            // Register with the parent
-            try {
-                if( host == null ) {
-                    // global
-                    parent=new ObjectName(domain +":type=Engine");
-                } else if( path==null ) {
-                    parent=new ObjectName(domain +
-                            ":type=Host,host=" + host);
-                } else {
-                    parent=new ObjectName(domain +":j2eeType=WebModule,name=//" +
-                            host + path);
-                }
-                if( mserver.isRegistered(parent ))  {
-                    if(log.isDebugEnabled())
-                        log.debug("Register with " + parent);
-                    mserver.setAttribute(parent, new Attribute("realm", this));
-                }
-            } catch (Exception e) {
-                log.error("Parent not available yet: " + parent);  
-            }
-        }
-        
-        if( oname==null ) {
-            // register
-            try {
-                ContainerBase cb=(ContainerBase)container;
-                oname=new ObjectName(cb.getDomain()+":type=Realm" +
-                        getRealmSuffix() + cb.getContainerSuffix());
-                Registry.getRegistry(null, null).registerComponent(this, oname, null );
-                if(log.isDebugEnabled())
-                    log.debug("Register Realm "+oname);
-            } catch (Throwable e) {
-                log.error( "Can't register " + oname, e);
-            }
-        }
-
-    }
-
-
     protected String getRealmSuffix() {
         return ",realmPath=" + getRealmPath();
     }
@@ -1440,6 +1392,7 @@ public abstract class RealmBase
             this.name = name;
         }
         
+        @Override
         public boolean equals(Object o)
         {
             boolean equals = false;
@@ -1450,10 +1403,12 @@ public abstract class RealmBase
             }
             return equals;
         }
+        @Override
         public int hashCode()
         {
             return name.hashCode();
         }
+        @Override
         public String toString()
         {
             return name;
