@@ -25,7 +25,7 @@ package org.apache.catalina.security;
  *
  * @author Glenn L. Nielsen
  * @author Jean-Francois Arcand
- * @version $Id: SecurityClassLoad.java 952026 2010-06-06 23:56:45Z markt $
+ * @version $Id: SecurityClassLoad.java 1133586 2011-06-08 22:20:22Z markt $
  */
 
 public final class SecurityClassLoad {
@@ -38,95 +38,124 @@ public final class SecurityClassLoad {
         }
         
         loadCorePackage(loader);
+        loadCoyotePackage(loader);
         loadLoaderPackage(loader);
+        loadRealmPackage(loader);
         loadSessionPackage(loader);
         loadUtilPackage(loader);
         loadJavaxPackage(loader);
-        loadCoyotePackage(loader);
-        loadHttp11Package(loader);
+        loadConnectorPackage(loader);        
         loadTomcatPackage(loader);
     }
     
     
-    private final static void loadCorePackage(ClassLoader loader)
+    private static final void loadCorePackage(ClassLoader loader)
         throws Exception {
-        String basePackage = "org.apache.catalina.";
+        final String basePackage = "org.apache.catalina.core.";
         loader.loadClass
             (basePackage +
-             "core.ApplicationContextFacade$1");
+             "ApplicationContextFacade$1");
         loader.loadClass
             (basePackage +
-             "core.ApplicationDispatcher$PrivilegedForward");
+             "ApplicationDispatcher$PrivilegedForward");
         loader.loadClass
             (basePackage +
-             "core.ApplicationDispatcher$PrivilegedInclude");
+             "ApplicationDispatcher$PrivilegedInclude");
         loader.loadClass
             (basePackage +
-             "core.ContainerBase$PrivilegedAddChild");
+            "AsyncContextImpl");
         loader.loadClass
             (basePackage +
-             "core.StandardWrapper$1");
+            "AsyncContextImpl$DebugException");
         loader.loadClass
             (basePackage +
-              "core.ApplicationHttpRequest$AttributeNamesEnumerator");
+            "AsyncContextImpl$1");
+        loader.loadClass
+            (basePackage +
+            "AsyncListenerWrapper");
+        loader.loadClass
+            (basePackage +
+             "ContainerBase$PrivilegedAddChild");
+        loader.loadClass
+            (basePackage +
+             "DefaultInstanceManager$1");
+        loader.loadClass
+            (basePackage +
+             "DefaultInstanceManager$2");
+        loader.loadClass
+            (basePackage +
+             "DefaultInstanceManager$3");
+        loader.loadClass
+            (basePackage +
+             "DefaultInstanceManager$4");
+        loader.loadClass
+            (basePackage +
+             "DefaultInstanceManager$5");
+        loader.loadClass
+            (basePackage +
+             "ApplicationHttpRequest$AttributeNamesEnumerator");
     }
     
     
-    private final static void loadLoaderPackage(ClassLoader loader)
+    private static final void loadLoaderPackage(ClassLoader loader)
         throws Exception {
-        String basePackage = "org.apache.catalina.";
+        final String basePackage = "org.apache.catalina.loader.";
         loader.loadClass
             (basePackage +
-             "loader.WebappClassLoader$PrivilegedFindResourceByName");
+             "WebappClassLoader$PrivilegedFindResourceByName");
     }
     
     
-    private final static void loadSessionPackage(ClassLoader loader)
+    private static final void loadRealmPackage(ClassLoader loader)
+            throws Exception {
+        final String basePackage = "org.apache.catalina.realm.";
+        loader.loadClass
+            (basePackage + "LockOutRealm$LockRecord");
+    }
+
+
+    private static final void loadSessionPackage(ClassLoader loader)
         throws Exception {
-        String basePackage = "org.apache.catalina.";
+        final String basePackage = "org.apache.catalina.session.";
         loader.loadClass
-            (basePackage + "session.StandardSession");
+            (basePackage + "StandardSession");
         loader.loadClass
-            (basePackage + "session.StandardSession$PrivilegedSetTccl");
+            (basePackage + "StandardSession$PrivilegedSetTccl");
         loader.loadClass
-            (basePackage +
-             "session.StandardSession$1");
+            (basePackage + "StandardSession$1");
         loader.loadClass
-            (basePackage +
-             "session.StandardManager$PrivilegedDoUnload");
+            (basePackage + "StandardManager$PrivilegedDoUnload");
     }
     
     
-    private final static void loadUtilPackage(ClassLoader loader)
+    private static final void loadUtilPackage(ClassLoader loader)
         throws Exception {
-        String basePackage = "org.apache.catalina.";
-        loader.loadClass
-            (basePackage + "util.URL");
-        loader.loadClass(basePackage + "util.Enumerator");
-        loader.loadClass(basePackage + "util.DefaultAnnotationProcessor$1");
-        loader.loadClass(basePackage + "util.DefaultAnnotationProcessor$2");
-        loader.loadClass(basePackage + "util.DefaultAnnotationProcessor$3");
-        loader.loadClass(basePackage + "util.DefaultAnnotationProcessor$4");
+        final String basePackage = "org.apache.catalina.util.";
+        loader.loadClass(basePackage + "Enumerator");
+        loader.loadClass(basePackage + "ParameterMap");
     }
     
     
-    private final static void loadJavaxPackage(ClassLoader loader)
+    private static final void loadCoyotePackage(ClassLoader loader)
+            throws Exception {
+        final String basePackage = "org.apache.coyote.";
+        loader.loadClass(basePackage + "http11.AbstractOutputBuffer$1");
+        loader.loadClass(basePackage + "http11.Constants");
+        // Make sure system property is read at this point
+        Class<?> clazz = loader.loadClass(basePackage + "Constants");
+        clazz.newInstance();
+    }
+
+
+    private static final void loadJavaxPackage(ClassLoader loader)
         throws Exception {
         loader.loadClass("javax.servlet.http.Cookie");
     }
     
 
-    private final static void loadHttp11Package(ClassLoader loader)
+    private static final void loadConnectorPackage(ClassLoader loader)
         throws Exception {
-        String basePackage = "org.apache.coyote.http11.";
-        loader.loadClass(basePackage + "InternalOutputBuffer$1");
-        loader.loadClass(basePackage + "InternalOutputBuffer$2");
-    }
-    
-    
-    private final static void loadCoyotePackage(ClassLoader loader)
-        throws Exception {
-        String basePackage = "org.apache.catalina.connector.";
+        final String basePackage = "org.apache.catalina.connector.";
         loader.loadClass
             (basePackage +
              "RequestFacade$GetAttributePrivilegedAction");
@@ -207,14 +236,30 @@ public final class SecurityClassLoad {
              "Response$3");
     }
 
-    private final static void loadTomcatPackage(ClassLoader loader)
+    private static final void loadTomcatPackage(ClassLoader loader)
         throws Exception {
-        String basePackage = "org.apache.tomcat.";
-        loader.loadClass(basePackage + "util.net.SSLSupport$CipherData");
+        final String basePackage = "org.apache.tomcat.";
+        loader.loadClass(basePackage + "util.buf.StringCache");
+        loader.loadClass(basePackage + "util.buf.StringCache$ByteEntry");
+        loader.loadClass(basePackage + "util.buf.StringCache$CharEntry");
+        loader.loadClass(basePackage + "util.http.HttpMessages");
         // Make sure system property is read at this point
         Class<?> clazz = loader.loadClass(
                 basePackage + "util.http.FastHttpDateFormat");
         clazz.newInstance();
+        loader.loadClass(basePackage + "util.http.HttpMessages");
+        loader.loadClass(basePackage + "util.net.Constants");
+        loader.loadClass(basePackage +
+                "util.net.NioBlockingSelector$BlockPoller$1");
+        loader.loadClass(basePackage +
+                "util.net.NioBlockingSelector$BlockPoller$2");
+        loader.loadClass(basePackage +
+                "util.net.NioBlockingSelector$BlockPoller$3");
+        loader.loadClass(basePackage + "util.net.SSLSupport$CipherData");
+        loader.loadClass
+            (basePackage + "util.net.JIoEndpoint$PrivilegedSetTccl");
+        loader.loadClass
+            (basePackage + "util.net.AprEndpoint$PrivilegedSetTccl");
     }
 }
 
