@@ -30,19 +30,21 @@ import org.apache.catalina.tribes.ChannelMessage;
 import org.apache.catalina.tribes.ChannelReceiver;
 import org.apache.catalina.tribes.ChannelSender;
 import org.apache.catalina.tribes.ErrorHandler;
+import org.apache.catalina.tribes.Heartbeat;
 import org.apache.catalina.tribes.ManagedChannel;
 import org.apache.catalina.tribes.Member;
 import org.apache.catalina.tribes.MembershipListener;
 import org.apache.catalina.tribes.MembershipService;
+import org.apache.catalina.tribes.RemoteProcessException;
+import org.apache.catalina.tribes.UniqueId;
 import org.apache.catalina.tribes.group.interceptors.MessageDispatchInterceptor;
+import org.apache.catalina.tribes.io.BufferPool;
 import org.apache.catalina.tribes.io.ChannelData;
 import org.apache.catalina.tribes.io.XByteBuffer;
-import org.apache.catalina.tribes.UniqueId;
-import org.apache.catalina.tribes.Heartbeat;
-import org.apache.catalina.tribes.io.BufferPool;
-import org.apache.catalina.tribes.RemoteProcessException;
-import org.apache.catalina.tribes.util.Logs;
 import org.apache.catalina.tribes.util.Arrays;
+import org.apache.catalina.tribes.util.Logs;
+import org.apache.juli.logging.Log;
+import org.apache.juli.logging.LogFactory;
 
 /**
  * The default implementation of a Channel.<br>
@@ -51,9 +53,11 @@ import org.apache.catalina.tribes.util.Arrays;
  * The channel has an chain of interceptors that can modify the message or perform other logic.<br>
  * It manages a complete group, both membership and replication.
  * @author Filip Hanik
- * @version $Revision: 568742 $, $Date: 2007-08-22 22:19:54 +0200 (Mi, 22. Aug 2007) $
+ * @version $Id: GroupChannel.java 987920 2010-08-22 15:34:34Z markt $
  */
 public class GroupChannel extends ChannelInterceptorBase implements ManagedChannel {
+    private static final Log log = LogFactory.getLog(GroupChannel.class);
+
     /**
      * Flag to determine if the channel manages its own heartbeat
      * If set to true, the channel will start a local thread for the heart beat.
@@ -79,7 +83,7 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
     protected ChannelCoordinator coordinator = new ChannelCoordinator();
 
     /**
-     * The first interceptor in the inteceptor stack.
+     * The first interceptor in the interceptor stack.
      * The interceptors are chained in a linked list, so we only need a reference to the
      * first one
      */
@@ -88,12 +92,12 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
     /**
      * A list of membership listeners that subscribe to membership announcements
      */
-    protected ArrayList membershipListeners = new ArrayList();
+    protected ArrayList<Object> membershipListeners = new ArrayList<Object>();
 
     /**
      * A list of channel listeners that subscribe to incoming messages
      */
-    protected ArrayList channelListeners = new ArrayList();
+    protected ArrayList<Object> channelListeners = new ArrayList<Object>();
 
     /**
      * If set to true, the GroupChannel will check to make sure that
@@ -145,9 +149,10 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
      * Invoke this method from the application on a periodic basis if
      * you have turned off internal heartbeats <code>channel.setHeartbeat(false)</code>
      */
+    @Override
     public void heartbeat() {
         super.heartbeat();
-        Iterator i = membershipListeners.iterator();
+        Iterator<Object> i = membershipListeners.iterator();
         while ( i.hasNext() ) {
             Object o = i.next();
             if ( o instanceof Heartbeat ) ((Heartbeat)o).heartbeat();
@@ -237,6 +242,7 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
      * but its an extreme case, and you're probably better off doing that logic between the applications itself.
      * @param msg ChannelMessage
      */
+    @Override
     public void messageReceived(ChannelMessage msg) {
         if ( msg == null ) return;
         try {
@@ -313,6 +319,7 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
      * and the channel will broadcast it to the membership listeners
      * @param member Member - the new member
      */
+    @Override
     public void memberAdded(Member member) {
         //notify upwards
         for (int i=0; i<membershipListeners.size(); i++ ) {
@@ -326,6 +333,7 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
      * and the channel will broadcast it to the membership listeners
      * @param member Member - the member that left or crashed
      */
+    @Override
     public void memberDisappeared(Member member) {
         //notify upwards
         for (int i=0; i<membershipListeners.size(); i++ ) {
@@ -344,7 +352,7 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
         if ( getFirstInterceptor() != null &&
              ((getFirstInterceptor().getNext() instanceof ChannelCoordinator))) {
             ChannelInterceptor interceptor = null;
-            Class clazz = null;
+            Class<?> clazz = null;
             try {
                 clazz = Class.forName("org.apache.catalina.tribes.group.interceptors.MessageDispatch15Interceptor",
                                       true,GroupChannel.class.getClassLoader());
@@ -367,7 +375,7 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
      * @throws ChannelException
      */
     protected void checkOptionFlags() throws ChannelException {
-        StringBuffer conflicts = new StringBuffer();
+        StringBuilder conflicts = new StringBuilder();
         ChannelInterceptor first = interceptors;
         while ( first != null ) {
             int flag = first.getOptionFlag();
@@ -401,6 +409,7 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
      * @throws ChannelException
      * @see org.apache.catalina.tribes.Channel#start(int)
      */
+    @Override
     public synchronized void start(int svc) throws ChannelException {
         setupDefaultStack();
         if (optionCheck) checkOptionFlags();
@@ -417,6 +426,7 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
      * @throws ChannelException
      * @see org.apache.catalina.tribes.Channel#stop(int)
      */
+    @Override
     public synchronized void stop(int svc) throws ChannelException {
         if (hbthread != null) {
             hbthread.stopHeartbeat();
@@ -529,7 +539,7 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
      * Returns an iterator of all the interceptors in this stack
      * @return Iterator
      */
-    public Iterator getInterceptors() {
+    public Iterator<ChannelInterceptor> getInterceptors() {
         return new InterceptorIterator(this.getNext(),this.coordinator);
     }
 
@@ -581,7 +591,7 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
 
     /**
      * Returns the sleep time in milliseconds that the internal heartbeat will
-     * sleep in between invokations of <code>Channel.heartbeat()</code>
+     * sleep in between invocations of <code>Channel.heartbeat()</code>
      * @return long
      */
     public long getHeartbeatSleeptime() {
@@ -596,7 +606,7 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
      *
      * @version 1.0
      */
-    public static class InterceptorIterator implements Iterator {
+    public static class InterceptorIterator implements Iterator<ChannelInterceptor> {
         private ChannelInterceptor end;
         private ChannelInterceptor start;
         public InterceptorIterator(ChannelInterceptor start, ChannelInterceptor end) {
@@ -608,8 +618,8 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
             return start!=null && start != end;
         }
 
-        public Object next() {
-            Object result = null;
+        public ChannelInterceptor next() {
+            ChannelInterceptor result = null;
             if ( hasNext() ) {
                 result = start;
                 start = start.getNext();
@@ -632,13 +642,13 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
      * @version 1.0
      */
     public static class HeartbeatThread extends Thread {
-        protected static org.apache.juli.logging.Log log = org.apache.juli.logging.LogFactory.getLog(HeartbeatThread.class);
+        private static final Log log = LogFactory.getLog(HeartbeatThread.class);
         protected static int counter = 1;
         protected static synchronized int inc() {
             return counter++;
         }
 
-        protected boolean doRun = true;
+        protected volatile boolean doRun = true;
         protected GroupChannel channel;
         protected long sleepTime;
         public HeartbeatThread(GroupChannel channel, long sleepTime) {
@@ -654,6 +664,7 @@ public class GroupChannel extends ChannelInterceptorBase implements ManagedChann
             interrupt();
         }
 
+        @Override
         public void run() {
             while (doRun) {
                 try {
