@@ -23,18 +23,30 @@ import java.text.SimpleDateFormat;
 import java.util.Map;
 import java.util.TimeZone;
 
+import org.apache.juli.logging.Log;
+import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.res.StringManager;
+
 
 /**
  * General purpose request parsing and encoding utility methods.
  *
  * @author Craig R. McClanahan
  * @author Tim Tye
- * @version $Id: RequestUtil.java 939353 2010-04-29 15:50:43Z kkolinko $
+ * @version $Id: RequestUtil.java 1069170 2011-02-09 23:41:32Z markt $
  */
 
 public final class RequestUtil {
 
 
+    private static final Log log = LogFactory.getLog(RequestUtil.class);
+
+    /**
+     * The string resources for this package.
+     */
+    private static final StringManager sm =
+        StringManager.getManager("org.apache.catalina.util");
+    
     /**
      * The DateFormat to use for generating readable dates in cookies.
      */
@@ -60,7 +72,7 @@ public final class RequestUtil {
 
         char content[] = new char[message.length()];
         message.getChars(0, message.length(), content, 0);
-        StringBuffer result = new StringBuffer(content.length + 50);
+        StringBuilder result = new StringBuilder(content.length + 50);
         for (int i = 0; i < content.length; i++) {
             switch (content[i]) {
             case '<':
@@ -172,11 +184,12 @@ public final class RequestUtil {
      *
      * @param map Map that accumulates the resulting parameters
      * @param data Input string containing request parameters
-     *
-     * @exception IllegalArgumentException if the data is malformed
+     * @param encoding The encoding to use; if null, the default encoding is
+     * used. If an unsupported encoding is specified the parameters will not be
+     * parsed and the map will not be modified
      */
-    public static void parseParameters(Map map, String data, String encoding)
-        throws UnsupportedEncodingException {
+    public static void parseParameters(Map<String,String[]> map, String data,
+            String encoding) {
 
         if ((data != null) && (data.length() > 0)) {
 
@@ -190,10 +203,12 @@ public final class RequestUtil {
                 } else {
                     bytes = data.getBytes(encoding);
                 }
+                parseParameters(map, bytes, encoding);
             } catch (UnsupportedEncodingException uee) {
+                log.debug(sm.getString("requestUtil.parseParameters.uee",
+                        encoding), uee);
             }
 
-            parseParameters(map, bytes, encoding);
         }
 
     }
@@ -220,7 +235,8 @@ public final class RequestUtil {
      * string is not a query string.
      *
      * @param str The url-encoded string
-     * @param enc The encoding to use; if null, the default encoding is used
+     * @param enc The encoding to use; if null, the default encoding is used. If
+     * an unsupported encoding is specified null will be returned
      * @exception IllegalArgumentException if a '%' character is not followed
      * by a valid 2-digit hexadecimal number
      */
@@ -232,7 +248,8 @@ public final class RequestUtil {
      * Decode and return the specified URL-encoded String.
      *
      * @param str The url-encoded string
-     * @param enc The encoding to use; if null, the default encoding is used
+     * @param enc The encoding to use; if null, the default encoding is used. If
+     * an unsupported encoding is specified null will be returned
      * @param isQuery Is this a query string being processed
      * @exception IllegalArgumentException if a '%' character is not followed
      * by a valid 2-digit hexadecimal number
@@ -251,7 +268,9 @@ public final class RequestUtil {
             } else {
                 bytes = str.getBytes(enc);
             }
-        } catch (UnsupportedEncodingException uee) {}
+        } catch (UnsupportedEncodingException uee) {
+            log.debug(sm.getString("requestUtil.urlDecode.uee", enc), uee);
+        }
 
         return URLDecode(bytes, enc, isQuery);
 
@@ -288,7 +307,8 @@ public final class RequestUtil {
      * Decode and return the specified URL-encoded byte array.
      *
      * @param bytes The url-encoded byte array
-     * @param enc The encoding to use; if null, the default encoding is used
+     * @param enc The encoding to use; if null, the default encoding is used. If
+     * an unsupported encoding is specified null will be returned
      * @param isQuery Is this a query string being processed
      * @exception IllegalArgumentException if a '%' character is not followed
      * by a valid 2-digit hexadecimal number
@@ -296,7 +316,7 @@ public final class RequestUtil {
     public static String URLDecode(byte[] bytes, String enc, boolean isQuery) {
     
         if (bytes == null)
-            return (null);
+            return null;
 
         int len = bytes.length;
         int ix = 0;
@@ -306,6 +326,10 @@ public final class RequestUtil {
             if (b == '+' && isQuery) {
                 b = (byte)' ';
             } else if (b == '%') {
+                if (ix + 2 > len) {
+                    throw new IllegalArgumentException(
+                            sm.getString("requestUtil.urlDecode.missingDigit"));
+                }
                 b = (byte) ((convertHexDigit(bytes[ix++]) << 4)
                             + convertHexDigit(bytes[ix++]));
             }
@@ -314,8 +338,9 @@ public final class RequestUtil {
         if (enc != null) {
             try {
                 return new String(bytes, 0, ox, enc);
-            } catch (Exception e) {
-                e.printStackTrace();
+            } catch (UnsupportedEncodingException uee) {
+                log.debug(sm.getString("requestUtil.urlDecode.uee", enc), uee);
+                return null;
             }
         }
         return new String(bytes, 0, ox);
@@ -324,7 +349,7 @@ public final class RequestUtil {
 
 
     /**
-     * Convert a byte character value to hexidecimal digit value.
+     * Convert a byte character value to hexadecimal digit value.
      *
      * @param b the character value byte
      */
@@ -332,7 +357,9 @@ public final class RequestUtil {
         if ((b >= '0') && (b <= '9')) return (byte)(b - '0');
         if ((b >= 'a') && (b <= 'f')) return (byte)(b - 'a' + 10);
         if ((b >= 'A') && (b <= 'F')) return (byte)(b - 'A' + 10);
-        return 0;
+        throw new IllegalArgumentException(
+                sm.getString("requestUtil.convertHexDigit.notHex",
+                        Character.valueOf((char)b)));
     }
 
 
@@ -344,9 +371,10 @@ public final class RequestUtil {
      * @param name The parameter name
      * @param value The parameter value
      */
-    private static void putMapEntry( Map map, String name, String value) {
+    private static void putMapEntry( Map<String,String[]> map, String name,
+            String value) {
         String[] newValues = null;
-        String[] oldValues = (String[]) map.get(name);
+        String[] oldValues = map.get(name);
         if (oldValues == null) {
             newValues = new String[1];
             newValues[0] = value;
@@ -374,12 +402,14 @@ public final class RequestUtil {
      *
      * @param map Map that accumulates the resulting parameters
      * @param data Input string containing request parameters
-     * @param encoding Encoding to use for converting hex
+     * @param encoding The encoding to use; if null, the default encoding is
+     * used
      *
-     * @exception UnsupportedEncodingException if the data is malformed
+     * @exception UnsupportedEncodingException if the requested encoding is not
+     * supported.
      */
-    public static void parseParameters(Map map, byte[] data, String encoding)
-        throws UnsupportedEncodingException {
+    public static void parseParameters(Map<String,String[]> map, byte[] data,
+            String encoding) throws UnsupportedEncodingException {
 
         if (data != null && data.length > 0) {
             int    ix = 0;
