@@ -19,9 +19,12 @@ package org.apache.jasper;
 
 import java.io.File;
 import java.io.FileNotFoundException;
+import java.io.IOException;
+import java.net.JarURLConnection;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.net.URLClassLoader;
+import java.net.URLConnection;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Set;
@@ -30,12 +33,16 @@ import javax.servlet.ServletContext;
 import javax.servlet.jsp.tagext.TagInfo;
 
 import org.apache.jasper.compiler.Compiler;
+import org.apache.jasper.compiler.JarResource;
 import org.apache.jasper.compiler.JspRuntimeContext;
 import org.apache.jasper.compiler.JspUtil;
 import org.apache.jasper.compiler.Localizer;
 import org.apache.jasper.compiler.ServletWriter;
+import org.apache.jasper.compiler.TldLocation;
 import org.apache.jasper.servlet.JasperLoader;
 import org.apache.jasper.servlet.JspServletWrapper;
+import org.apache.juli.logging.Log;
+import org.apache.juli.logging.LogFactory;
 
 /**
  * A place holder for various things that are used through out the JSP
@@ -53,21 +60,17 @@ import org.apache.jasper.servlet.JspServletWrapper;
  */
 public class JspCompilationContext {
 
-    protected org.apache.juli.logging.Log log =
-        org.apache.juli.logging.LogFactory.getLog(JspCompilationContext.class);
+    private final Log log = LogFactory.getLog(JspCompilationContext.class); // must not be static
 
-    protected Map<String, URL> tagFileJarUrls;
-    protected boolean isPackagedTagFile;
+    protected Map<String, JarResource> tagFileJarUrls;
 
     protected String className;
     protected String jspUri;
-    protected boolean isErrPage;
     protected String basePackageName;
     protected String derivedPackageName;
     protected String servletJavaFileName;
     protected String javaPath;
     protected String classFileName;
-    protected String contentType;
     protected ServletWriter writer;
     protected Options options;
     protected JspServletWrapper jsw;
@@ -77,7 +80,7 @@ public class JspCompilationContext {
     protected String baseURI;
     protected String outputDir;
     protected ServletContext context;
-    protected URLClassLoader loader;
+    protected ClassLoader loader;
 
     protected JspRuntimeContext rctxt;
 
@@ -85,23 +88,21 @@ public class JspCompilationContext {
 
     protected URLClassLoader jspLoader;
     protected URL baseUrl;
-    protected Class servletClass;
+    protected Class<?> servletClass;
 
     protected boolean isTagFile;
     protected boolean protoTypeMode;
     protected TagInfo tagInfo;
-    protected URL tagFileJarUrl;
+    protected JarResource tagJarResource;
 
     // jspURI _must_ be relative to the context
     public JspCompilationContext(String jspUri,
-                                 boolean isErrPage,
                                  Options options,
                                  ServletContext context,
                                  JspServletWrapper jsw,
                                  JspRuntimeContext rctxt) {
 
         this.jspUri = canonicalURI(jspUri);
-        this.isErrPage = isErrPage;
         this.options = options;
         this.jsw = jsw;
         this.context = context;
@@ -111,7 +112,7 @@ public class JspCompilationContext {
         if (baseURI == null) {
             baseURI = "/";
         } else if (baseURI.charAt(0) != '/') {
-            // strip the basde slash since it will be combined with the
+            // strip the base slash since it will be combined with the
             // uriBase to generate a file
             baseURI = "/" + baseURI;
         }
@@ -120,7 +121,7 @@ public class JspCompilationContext {
         }
 
         this.rctxt = rctxt;
-        this.tagFileJarUrls = new HashMap<String, URL>();
+        this.tagFileJarUrls = new HashMap<String, JarResource>();
         this.basePackageName = Constants.JSP_PACKAGE_NAME;
     }
 
@@ -130,14 +131,11 @@ public class JspCompilationContext {
                                  ServletContext context,
                                  JspServletWrapper jsw,
                                  JspRuntimeContext rctxt,
-                                 URL tagFileJarUrl) {
-        this(tagfile, false, options, context, jsw, rctxt);
+                                 JarResource tagJarResource) {
+        this(tagfile, options, context, jsw, rctxt);
         this.isTagFile = true;
         this.tagInfo = tagInfo;
-        this.tagFileJarUrl = tagFileJarUrl;
-        if (tagFileJarUrl != null) {
-            isPackagedTagFile = true;
-        }
+        this.tagJarResource = tagJarResource;
     }
 
     /* ==================== Methods to override ==================== */
@@ -170,7 +168,7 @@ public class JspCompilationContext {
         return rctxt.getParentClassLoader();
     }
 
-    public void setClassLoader(URLClassLoader loader) {
+    public void setClassLoader(ClassLoader loader) {
         this.loader = loader;
     }
 
@@ -179,8 +177,7 @@ public class JspCompilationContext {
             jspLoader = new JasperLoader
             (new URL[] {baseUrl},
                     getClassLoader(),
-                    rctxt.getPermissionCollection(),
-                    rctxt.getCodeSource());
+                    rctxt.getPermissionCollection());
         }
         return jspLoader;
     }
@@ -193,9 +190,9 @@ public class JspCompilationContext {
      * plus the directory derived from the package name.
      */
     public String getOutputDir() {
-	if (outputDir == null) {
-	    createOutputDir();
-	}
+        if (outputDir == null) {
+            createOutputDir();
+        }
 
         return outputDir;
     }
@@ -205,7 +202,7 @@ public class JspCompilationContext {
      * is not done yet. Right now we're just hardcoding the actual
      * compilers that are created. 
      */
-    public Compiler createCompiler() throws JasperException {
+    public Compiler createCompiler() {
         if (jspCompiler != null ) {
             return jspCompiler;
         }
@@ -264,7 +261,7 @@ public class JspCompilationContext {
      */
     public String resolveRelativeUri(String uri) {
         // sometimes we get uri's massaged from File(String), so check for
-        // a root directory deperator char
+        // a root directory separator char
         if (uri.startsWith("/") || uri.startsWith(File.separator)) {
             return uri;
         } else {
@@ -288,12 +285,12 @@ public class JspCompilationContext {
 
         if (res.startsWith("/META-INF/")) {
             // This is a tag file packaged in a jar that is being compiled
-            URL jarUrl = tagFileJarUrls.get(res);
-            if (jarUrl == null) {
-                jarUrl = tagFileJarUrl;
+            JarResource jarResource = tagFileJarUrls.get(res);
+            if (jarResource == null) {
+                jarResource = tagJarResource;
             }
-            if (jarUrl != null) {
-                result = new URL(jarUrl.toExternalForm() + res.substring(1));
+            if (jarResource != null) {
+                result = jarResource.getEntry(res.substring(1));
             } else {
                 // May not be in a JAR in some IDE environments
                 result = context.getResource(canonicalURI(res));
@@ -310,7 +307,7 @@ public class JspCompilationContext {
     }
 
 
-    public Set getResourcePaths(String path) {
+    public Set<String> getResourcePaths(String path) {
         return context.getResourcePaths(canonicalURI(path));
     }
 
@@ -333,12 +330,12 @@ public class JspCompilationContext {
      * The map is populated when parsing the tag-file elements of the TLDs
      * of any imported taglibs. 
      */
-    public URL getTagFileJarUrl(String tagFile) {
+    public JarResource getTagFileJarResource(String tagFile) {
         return this.tagFileJarUrls.get(tagFile);
     }
 
-    public void setTagFileJarUrl(String tagFile, URL tagFileURL) {
-        this.tagFileJarUrls.put(tagFile, tagFileURL);
+    public void setTagFileJarResource(String tagFile, JarResource jarResource) {
+        this.tagFileJarUrls.put(tagFile, jarResource);
     }
 
     /**
@@ -347,8 +344,8 @@ public class JspCompilationContext {
      * JspCompilationContext does not correspond to a tag file, or if the
      * corresponding tag file is not packaged in a JAR.
      */
-    public URL getTagFileJarUrl() {
-        return this.tagFileJarUrl;
+    public JarResource getTagFileJarResource() {
+        return this.tagJarResource;
     }
 
     /* ==================== Common implementation ==================== */
@@ -388,16 +385,41 @@ public class JspCompilationContext {
         return jspUri;
     }
 
-    /**
-     * Are we processing something that has been declared as an
-     * errorpage? 
-     */
-    public boolean isErrorPage() {
-        return isErrPage;
-    }
-
-    public void setErrorPage(boolean isErrPage) {
-        this.isErrPage = isErrPage;
+    public long getJspLastModified() {
+        long result = -1;
+        URLConnection uc = null;
+        try {
+            URL jspUrl = getResource(getJspFile());
+            if (jspUrl == null) {
+                incrementRemoved();
+                return result;
+            }
+            uc = jspUrl.openConnection();
+            if (uc instanceof JarURLConnection) {
+                result = ((JarURLConnection) uc).getJarEntry().getTime();
+            } else {
+                result = uc.getLastModified();
+            }
+        } catch (IOException e) {
+            if (log.isDebugEnabled()) {
+                log.debug(Localizer.getMessage(
+                        "jsp.error.lastModified", getJspFile()), e);
+            }
+            result = -1;
+        } finally {
+            if (uc != null) {
+                try {
+                    uc.getInputStream().close();
+                } catch (IOException e) {
+                    if (log.isDebugEnabled()) {
+                        log.debug(Localizer.getMessage(
+                                "jsp.error.lastModified", getJspFile()), e);
+                    }
+                    result = -1;
+                }
+            }
+        }
+        return result;
     }
 
     public boolean isTagFile() {
@@ -428,7 +450,7 @@ public class JspCompilationContext {
     /**
      * Package name for the generated class is make up of the base package
      * name, which is user settable, and the derived package name.  The
-     * derived package name directly mirrors the file heirachy of the JSP page.
+     * derived package name directly mirrors the file hierarchy of the JSP page.
      */
     public String getServletPackageName() {
         if (isTagFile()) {
@@ -456,7 +478,7 @@ public class JspCompilationContext {
         }
         return derivedPackageName;
     }
-	    
+            
     /**
      * The package name into which the servlet class is generated.
      */
@@ -500,12 +522,12 @@ public class JspCompilationContext {
         }
 
         if (isTagFile()) {
-	    String tagName = tagInfo.getTagClassName();
+            String tagName = tagInfo.getTagClassName();
             javaPath = tagName.replace('.', '/') + ".java";
         } else {
             javaPath = getServletPackageName().replace('.', '/') + '/' +
                        getServletClassName() + ".java";
-	}
+        }
         return javaPath;
     }
 
@@ -516,19 +538,6 @@ public class JspCompilationContext {
         return classFileName;
     }
 
-    /**
-     * Get the content type of this JSP.
-     *
-     * Content type includes content type and encoding.
-     */
-    public String getContentType() {
-        return contentType;
-    }
-
-    public void setContentType(String contentType) {
-        this.contentType = contentType;
-    }
-
     /**
      * Where is the servlet being generated?
      */
@@ -549,8 +558,8 @@ public class JspCompilationContext {
      * Returns null if the given uri is not associated with any tag library
      * 'exposed' in the web application.
      */
-    public String[] getTldLocation(String uri) throws JasperException {
-        String[] location = 
+    public TldLocation getTldLocation(String uri) throws JasperException {
+        TldLocation location = 
             getOptions().getTldLocationsCache().getLocation(uri);
         return location;
     }
@@ -613,18 +622,11 @@ public class JspCompilationContext {
 
     // ==================== Manipulating the class ====================
 
-    public Class load() 
-        throws JasperException, FileNotFoundException
-    {
+    public Class<?> load() throws JasperException {
         try {
             getJspLoader();
             
-            String name;
-            if (isTagFile()) {
-                name = tagInfo.getTagClassName();
-            } else {
-                name = getServletPackageName() + "." + getServletClassName();
-            }
+            String name = getFQCN();
             servletClass = jspLoader.loadClass(name);
         } catch (ClassNotFoundException cex) {
             throw new JasperException(Localizer.getMessage("jsp.error.unable.load"),
@@ -637,6 +639,16 @@ public class JspCompilationContext {
         return servletClass;
     }
 
+    public String getFQCN() {
+        String name;
+        if (isTagFile()) {
+            name = tagInfo.getTagClassName();
+        } else {
+            name = getServletPackageName() + "." + getServletClassName();
+        }
+        return name;
+    }
+
     // ==================== protected methods ==================== 
 
     static Object outputDirLock = new Object();
@@ -688,7 +700,7 @@ public class JspCompilationContext {
 
     protected static final String canonicalURI(String s) {
        if (s == null) return null;
-       StringBuffer result = new StringBuffer();
+       StringBuilder result = new StringBuilder();
        final int len = s.length();
        int pos = 0;
        while (pos < len) {
