@@ -17,12 +17,12 @@
 
 package org.apache.jasper.compiler;
 
+import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.InputStream;
 import java.io.PrintWriter;
 import java.io.StringWriter;
-import java.net.JarURLConnection;
 import java.net.URL;
 import java.util.Collection;
 import java.util.Enumeration;
@@ -30,8 +30,6 @@ import java.util.Hashtable;
 import java.util.Iterator;
 import java.util.Map;
 import java.util.Vector;
-import java.util.jar.JarFile;
-import java.util.zip.ZipEntry;
 
 import javax.servlet.jsp.tagext.FunctionInfo;
 import javax.servlet.jsp.tagext.PageData;
@@ -47,6 +45,7 @@ import javax.servlet.jsp.tagext.VariableInfo;
 
 import org.apache.jasper.JasperException;
 import org.apache.jasper.JspCompilationContext;
+import org.apache.jasper.util.ExceptionUtils;
 import org.apache.jasper.xmlparser.ParserUtils;
 import org.apache.jasper.xmlparser.TreeNode;
 import org.apache.juli.logging.Log;
@@ -64,7 +63,7 @@ import org.apache.juli.logging.LogFactory;
 class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
 
     // Logger
-    private Log log = LogFactory.getLog(TagLibraryInfoImpl.class);
+    private final Log log = LogFactory.getLog(TagLibraryInfoImpl.class);
 
     private JspCompilationContext ctxt;
     
@@ -82,6 +81,7 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
         }
     }
 
+    @Override
     public String toString() {
         StringWriter sw = new StringWriter();
         PrintWriter out = new PrintWriter(sw);
@@ -111,27 +111,31 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
     // the following is a workaround until these problems are resolved.
     private InputStream getResourceAsStream(String uri)
             throws FileNotFoundException {
-        try {
-            // see if file exists on the filesystem first
-            String real = ctxt.getRealPath(uri);
-            if (real == null) {
+        // Is uri absolute?
+        if (uri.startsWith("file:")) {
+            return new FileInputStream(new File(uri.substring(5)));
+        } else {
+            try {
+                // see if file exists on the filesystem
+                String real = ctxt.getRealPath(uri);
+                if (real == null) {
+                    return ctxt.getResourceAsStream(uri);
+                } else {
+                    return new FileInputStream(real);
+                }
+            } catch (FileNotFoundException ex) {
+                // if file not found on filesystem, get the resource through
+                // the context
                 return ctxt.getResourceAsStream(uri);
-            } else {
-                return new FileInputStream(real);
             }
-        } catch (FileNotFoundException ex) {
-            // if file not found on filesystem, get the resource through
-            // the context
-            return ctxt.getResourceAsStream(uri);
         }
-
     }
 
     /**
      * Constructor.
      */
     public TagLibraryInfoImpl(JspCompilationContext ctxt, ParserController pc,
-            PageInfo pi, String prefix, String uriIn, String[] location,
+            PageInfo pi, String prefix, String uriIn, TldLocation location,
             ErrorDispatcher err, Mark mark)
             throws JasperException {
         super(prefix, uriIn);
@@ -141,7 +145,6 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
         this.pi = pi;
         this.err = err;
         InputStream in = null;
-        JarFile jarFile = null;
 
         if (location == null) {
             // The URI points to the TLD itself or to a JAR file in which the
@@ -149,39 +152,34 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
             location = generateTLDLocation(uri, ctxt);
         }
 
+        String tldName = location.getName();
+        JarResource jarResource = location.getJarResource();
         try {
-            if (!location[0].endsWith("jar")) {
-                // Location points to TLD file
+            if (jarResource == null) {
+                // Location points directly to TLD file
                 try {
-                    in = getResourceAsStream(location[0]);
+                    in = getResourceAsStream(tldName);
                     if (in == null) {
-                        throw new FileNotFoundException(location[0]);
+                        throw new FileNotFoundException(tldName);
                     }
                 } catch (FileNotFoundException ex) {
-                    err.jspError(mark, "jsp.error.file.not.found", location[0]);
+                    err.jspError(mark, "jsp.error.file.not.found", tldName);
                 }
 
-                parseTLD(ctxt, location[0], in, null);
+                parseTLD(tldName, in, null);
                 // Add TLD to dependency list
                 PageInfo pageInfo = ctxt.createCompiler().getPageInfo();
                 if (pageInfo != null) {
-                    pageInfo.addDependant(location[0]);
+                    pageInfo.addDependant(tldName);
                 }
             } else {
                 // Tag library is packaged in JAR file
                 try {
-                    URL jarFileUrl = new URL("jar:" + location[0] + "!/");
-                    JarURLConnection conn = (JarURLConnection) jarFileUrl
-                            .openConnection();
-                    conn.setUseCaches(false);
-                    conn.connect();
-                    jarFile = conn.getJarFile();
-                    ZipEntry jarEntry = jarFile.getEntry(location[1]);
-                    in = jarFile.getInputStream(jarEntry);
-                    parseTLD(ctxt, location[0], in, jarFileUrl);
+                    in = jarResource.getEntry(tldName).openStream();
+                    parseTLD(jarResource.getUrl(), in, jarResource);
                 } catch (Exception ex) {
-                    err.jspError(mark, "jsp.error.tld.unable_to_read", location[0],
-                            location[1], ex.toString());
+                    err.jspError(mark, "jsp.error.tld.unable_to_read", jarResource.getUrl(),
+                            tldName, ex.toString());
                 }
             }
         } finally {
@@ -189,21 +187,17 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
                 try {
                     in.close();
                 } catch (Throwable t) {
-                }
-            }
-            if (jarFile != null) {
-                try {
-                    jarFile.close();
-                } catch (Throwable t) {
+                    ExceptionUtils.handleThrowable(t);
                 }
             }
         }
 
     }
 
+    @Override
     public TagLibraryInfo[] getTagLibraryInfos() {
-        Collection coll = pi.getTaglibs();
-        return (TagLibraryInfo[]) coll.toArray(new TagLibraryInfo[0]);
+        Collection<TagLibraryInfo> coll = pi.getTaglibs();
+        return coll.toArray(new TagLibraryInfo[0]);
     }
     
     /*
@@ -211,11 +205,11 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
      * in The TLD's input stream @param jarFileUrl The JAR file containing the
      * TLD, or null if the tag library is not packaged in a JAR
      */
-    private void parseTLD(JspCompilationContext ctxt, String uri,
-            InputStream in, URL jarFileUrl) throws JasperException {
-        Vector tagVector = new Vector();
-        Vector tagFileVector = new Vector();
-        Hashtable functionTable = new Hashtable();
+    private void parseTLD(String uri, InputStream in, JarResource jarResource)
+            throws JasperException {
+        Vector<TagInfo> tagVector = new Vector<TagInfo>();
+        Vector<TagFileInfo> tagFileVector = new Vector<TagFileInfo>();
+        Hashtable<String, FunctionInfo> functionTable = new Hashtable<String, FunctionInfo>();
 
         // Create an iterator over the child elements of our <taglib> element
         ParserUtils pu = new ParserUtils();
@@ -227,10 +221,10 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
         this.jspversion = tld.findAttribute("version");
 
         // Process each child element of our <taglib> element
-        Iterator list = tld.findChildren();
+        Iterator<TreeNode> list = tld.findChildren();
 
         while (list.hasNext()) {
-            TreeNode element = (TreeNode) list.next();
+            TreeNode element = list.next();
             String tname = element.getName();
 
             if ("tlibversion".equals(tname) // JSP 1.1
@@ -250,8 +244,8 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
             else if ("tag".equals(tname))
                 tagVector.addElement(createTagInfo(element, jspversion));
             else if ("tag-file".equals(tname)) {
-                TagFileInfo tagFileInfo = createTagFileInfo(element, uri,
-                        jarFileUrl);
+                TagFileInfo tagFileInfo = createTagFileInfo(element,
+                        jarResource);
                 tagFileVector.addElement(tagFileInfo);
             } else if ("function".equals(tname)) { // JSP2.0
                 FunctionInfo funcInfo = createFunctionInfo(element);
@@ -262,10 +256,10 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
 
                 }
                 functionTable.put(funcName, funcInfo);
-            } else if ("display-name".equals(tname) || // Ignored elements
+            } else if ("display-name".equals(tname) ||
                     "small-icon".equals(tname) || "large-icon".equals(tname)
                     || "listener".equals(tname)) {
-                ;
+                // Ignored elements
             } else if ("taglib-extension".equals(tname)) {
                 // Recognized but ignored
             } else {
@@ -294,9 +288,9 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
 
         this.functions = new FunctionInfo[functionTable.size()];
         int i = 0;
-        Enumeration enumeration = functionTable.elements();
+        Enumeration<FunctionInfo> enumeration = functionTable.elements();
         while (enumeration.hasMoreElements()) {
-            this.functions[i++] = (FunctionInfo) enumeration.nextElement();
+            this.functions[i++] = enumeration.nextElement();
         }
     }
 
@@ -308,7 +302,7 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
      * the name of the TLD entry in the jar file, which is hardcoded to
      * META-INF/taglib.tld.
      */
-    private String[] generateTLDLocation(String uri, JspCompilationContext ctxt)
+    private TldLocation generateTLDLocation(String uri, JspCompilationContext ctxt)
             throws JasperException {
 
         int uriType = TldLocationsCache.uriType(uri);
@@ -319,24 +313,21 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
             uri = ctxt.resolveRelativeUri(uri);
         }
 
-        String[] location = new String[2];
-        location[0] = uri;
-        if (location[0].endsWith("jar")) {
+        if (uri.endsWith(".jar")) {
             URL url = null;
             try {
-                url = ctxt.getResource(location[0]);
+                url = ctxt.getResource(uri);
             } catch (Exception ex) {
-                err.jspError("jsp.error.tld.unable_to_get_jar", location[0], ex
+                err.jspError("jsp.error.tld.unable_to_get_jar", uri, ex
                         .toString());
             }
             if (url == null) {
-                err.jspError("jsp.error.tld.missing_jar", location[0]);
+                err.jspError("jsp.error.tld.missing_jar", uri);
             }
-            location[0] = url.toString();
-            location[1] = "META-INF/taglib.tld";
+            return new TldLocation("META-INF/taglib.tld", url.toString());
+        } else {
+            return new TldLocation(uri);
         }
-
-        return location;
     }
 
     private TagInfo createTagInfo(TreeNode elem, String jspVersion)
@@ -359,11 +350,11 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
         String largeIcon = null;
         boolean dynamicAttributes = false;
 
-        Vector attributeVector = new Vector();
-        Vector variableVector = new Vector();
-        Iterator list = elem.findChildren();
+        Vector<TagAttributeInfo> attributeVector = new Vector<TagAttributeInfo>();
+        Vector<TagVariableInfo> variableVector = new Vector<TagVariableInfo>();
+        Iterator<TreeNode> list = elem.findChildren();
         while (list.hasNext()) {
-            TreeNode element = (TreeNode) list.next();
+            TreeNode element = list.next();
             String tname = element.getName();
 
             if ("name".equals(tname)) {
@@ -414,7 +405,8 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
         TagExtraInfo tei = null;
         if (teiClassName != null && !teiClassName.equals("")) {
             try {
-                Class teiClass = ctxt.getClassLoader().loadClass(teiClassName);
+                Class<?> teiClass =
+                    ctxt.getClassLoader().loadClass(teiClassName);
                 tei = (TagExtraInfo) teiClass.newInstance();
             } catch (Exception e) {
                 err.jspError("jsp.error.teiclass.instantiation", teiClassName,
@@ -444,17 +436,17 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
      * the TLD, in case the tag file is specified relative to it @param jarFile
      * The JAR file, in case the tag file is packaged in a JAR
      * 
-     * @return TagInfo correspoding to tag file directives
+     * @return TagInfo corresponding to tag file directives
      */
-    private TagFileInfo createTagFileInfo(TreeNode elem, String uri,
-            URL jarFileUrl) throws JasperException {
+    private TagFileInfo createTagFileInfo(TreeNode elem, JarResource jarResource)
+            throws JasperException {
 
         String name = null;
         String path = null;
 
-        Iterator list = elem.findChildren();
+        Iterator<TreeNode> list = elem.findChildren();
         while (list.hasNext()) {
-            TreeNode child = (TreeNode) list.next();
+            TreeNode child = list.next();
             String tname = child.getName();
             if ("name".equals(tname)) {
                 name = child.getBody();
@@ -481,13 +473,13 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
             // See https://issues.apache.org/bugzilla/show_bug.cgi?id=46471
             // This needs to be removed once all the broken code that depends on
             // it has been removed
-            ctxt.setTagFileJarUrl(path, jarFileUrl);
+            ctxt.setTagFileJarResource(path, jarResource);
         } else if (!path.startsWith("/WEB-INF/tags")) {
             err.jspError("jsp.error.tagfile.illegalPath", path);
         }
 
         TagInfo tagInfo = TagFileProcessor.parseTagFileDirectives(
-                parserController, name, path, jarFileUrl, this);
+                parserController, name, path, jarResource, this);
         return new TagFileInfo(name, path, tagInfo);
     }
 
@@ -496,11 +488,11 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
         String type = null;
         String expectedType = null;
         String methodSignature = null;
-        boolean required = false, rtexprvalue = false, reqTime = false, isFragment = false, deferredValue = false, deferredMethod = false;
+        boolean required = false, rtexprvalue = false, isFragment = false, deferredValue = false, deferredMethod = false;
 
-        Iterator list = elem.findChildren();
+        Iterator<TreeNode> list = elem.findChildren();
         while (list.hasNext()) {
-            TreeNode element = (TreeNode) list.next();
+            TreeNode element = list.next();
             String tname = element.getName();
 
             if ("name".equals(tname)) {
@@ -555,9 +547,8 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
                 } else {
                     methodSignature = "java.lang.Object method()";
                 }
-            } else if ("description".equals(tname) || // Ignored elements
-            false) {
-                ;
+            } else if ("description".equals(tname) || false) {
+                // Ignored elements
             } else {
                 if (log.isWarnEnabled()) {
                     log.warn(Localizer.getMessage(
@@ -597,9 +588,9 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
         boolean declare = true;
         int scope = VariableInfo.NESTED;
 
-        Iterator list = elem.findChildren();
+        Iterator<TreeNode> list = elem.findChildren();
         while (list.hasNext()) {
-            TreeNode element = (TreeNode) list.next();
+            TreeNode element = list.next();
             String tname = element.getName();
             if ("name-given".equals(tname))
                 nameGiven = element.getBody();
@@ -639,11 +630,11 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
             throws JasperException {
 
         String validatorClass = null;
-        Map initParams = new Hashtable();
+        Map<String,Object> initParams = new Hashtable<String,Object>();
 
-        Iterator list = elem.findChildren();
+        Iterator<TreeNode> list = elem.findChildren();
         while (list.hasNext()) {
-            TreeNode element = (TreeNode) list.next();
+            TreeNode element = list.next();
             String tname = element.getName();
             if ("validator-class".equals(tname))
                 validatorClass = element.getBody();
@@ -663,7 +654,7 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
         TagLibraryValidator tlv = null;
         if (validatorClass != null && !validatorClass.equals("")) {
             try {
-                Class tlvClass = ctxt.getClassLoader()
+                Class<?> tlvClass = ctxt.getClassLoader()
                         .loadClass(validatorClass);
                 tlv = (TagLibraryValidator) tlvClass.newInstance();
             } catch (Exception e) {
@@ -680,9 +671,9 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
     String[] createInitParam(TreeNode elem) {
         String[] initParam = new String[2];
 
-        Iterator list = elem.findChildren();
+        Iterator<TreeNode> list = elem.findChildren();
         while (list.hasNext()) {
-            TreeNode element = (TreeNode) list.next();
+            TreeNode element = list.next();
             String tname = element.getName();
             if ("param-name".equals(tname)) {
                 initParam[0] = element.getBody();
@@ -706,9 +697,9 @@ class TagLibraryInfoImpl extends TagLibraryInfo implements TagConstants {
         String klass = null;
         String signature = null;
 
-        Iterator list = elem.findChildren();
+        Iterator<TreeNode> list = elem.findChildren();
         while (list.hasNext()) {
-            TreeNode element = (TreeNode) list.next();
+            TreeNode element = list.next();
             String tname = element.getName();
 
             if ("name".equals(tname)) {
