@@ -56,7 +56,7 @@ import org.apache.juli.logging.LogFactory;
  *  - remove methods not used in tomcat and redundant/not very generic
  *  - must be created from the ManagedBean - I don't think there were any direct
  *    uses, but now it is required.
- *  - some of the gratuituous flexibility removed - instead this is more predictive and
+ *  - some of the gratuitous flexibility removed - instead this is more predictive and
  *    strict with the use cases.
  *  - all Method and metadata is stored in ManagedBean. BaseModelBMean and ManagedBean act
  *    like Object and Class. 
@@ -81,7 +81,7 @@ import org.apache.juli.logging.LogFactory;
  * Limitations:
  * <ul>
  * <li>Only managed resources of type <code>objectReference</code> are
- *     supportd.</li>
+ *     supported.</li>
  * <li>Caching of attribute values and operation results is not supported.
  *     All calls to <code>invoke()</code> are immediately executed.</li>
  * <li>Persistence of MBean attributes and operations is not supported.</li>
@@ -101,7 +101,7 @@ import org.apache.juli.logging.LogFactory;
  * @author Costin Manolache
  */
 public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBeanNotificationBroadcaster {
-    private static Log log = LogFactory.getLog(BaseModelMBean.class);
+    private static final Log log = LogFactory.getLog(BaseModelMBean.class);
 
     // ----------------------------------------------------------- Constructors
 
@@ -143,8 +143,8 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
 
     // --------------------------------------------------- DynamicMBean Methods
     // TODO: move to ManagedBean
-    static final Object[] NO_ARGS_PARAM=new Object[0];
-    static final Class[] NO_ARGS_PARAM_SIG=new Class[0];
+    static final Object[] NO_ARGS_PARAM = new Object[0];
+    static final Class<?>[] NO_ARGS_PARAM_SIG = new Class[0];
     
     protected String resourceType = null;
 
@@ -163,6 +163,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
      * @exception ReflectionException if a Java reflection exception
      *  occurs when invoking the getter
      */
+    @Override
     public Object getAttribute(String name)
         throws AttributeNotFoundException, MBeanException,
             ReflectionException {
@@ -180,7 +181,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
         Method m=managedBean.getGetter(name, this, resource);
         Object result = null;
         try {
-            Class declaring=m.getDeclaringClass();
+            Class<?> declaring = m.getDeclaringClass();
             // workaround for catalina weird mbeans - the declaring class is BaseModelMBean.
             // but this is the catalina class.
             if( declaring.isAssignableFrom(this.getClass()) ) {
@@ -217,6 +218,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
      *
      * @param names Names of the requested attributes
      */
+    @Override
     public AttributeList getAttributes(String names[]) {
 
         // Validate the input parameters
@@ -231,8 +233,8 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
             try {
                 response.add(new Attribute(names[i],getAttribute(names[i])));
             } catch (Exception e) {
-                ; // Not having a particular attribute in the response
-                ; // is the indication of a getter problem
+                // Not having a particular attribute in the response
+                // is the indication of a getter problem
             }
         }
         return (response);
@@ -246,6 +248,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
     /**
      * Return the <code>MBeanInfo</code> object for this MBean.
      */
+    @Override
     public MBeanInfo getMBeanInfo() {
         return managedBean.getMBeanInfo();
     }
@@ -270,6 +273,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
      * @exception ReflectioNException if a Java reflection exception
      *  occurs when invoking a method
      */
+    @Override
     public Object invoke(String name, Object params[], String signature[])
         throws MBeanException, ReflectionException 
     {
@@ -285,7 +289,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
                  "Method name is null");
 
         if( log.isDebugEnabled()) log.debug("Invoke " + name);
-	MethodKey mkey = new MethodKey(name, signature);
+
         Method method= managedBean.getInvoke(name, params, signature, this, resource);
         
         // Invoke the selected method on the appropriate object
@@ -322,7 +326,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
 
     }
 
-    static Class getAttributeClass(String signature)
+    static Class<?> getAttributeClass(String signature)
         throws ReflectionException
     {
         if (signature.equals(Boolean.TYPE.getName()))
@@ -370,6 +374,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
      * @exception ReflectionException if a Java reflection exception
      *  occurs when invoking the getter
      */
+    @Override
     public void setAttribute(Attribute attribute)
         throws AttributeNotFoundException, MBeanException,
         ReflectionException
@@ -439,11 +444,12 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
         }
         //attributes.put( name, value );
 //        if( source != null ) {
-//            // this mbean is asscoiated with a source - maybe we want to persist
+//            // this mbean is associated with a source - maybe we want to persist
 //            source.updateField(oname, name, value);
 //        }
     }
 
+    @Override
     public String toString() {
         if( resource==null ) 
             return "BaseModelMbean[" + resourceType + "]";
@@ -457,6 +463,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
      *
      * @return The list of attributes that were set and their new values
      */
+    @Override
     public AttributeList setAttributes(AttributeList attributes) {
         AttributeList response = new AttributeList();
 
@@ -467,14 +474,14 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
         // Prepare and return our response, eating all exceptions
         String names[] = new String[attributes.size()];
         int n = 0;
-        Iterator items = attributes.iterator();
+        Iterator<?> items = attributes.iterator();
         while (items.hasNext()) {
             Attribute item = (Attribute) items.next();
             names[n++] = item.getName();
             try {
                 setAttribute(item);
             } catch (Exception e) {
-                ; // Ignore all exceptions
+                // Ignore all exceptions
             }
         }
 
@@ -580,6 +587,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
      *
      * @exception IllegalArgumentException if the listener parameter is null
      */
+    @Override
     public void addAttributeChangeNotificationListener
         (NotificationListener listener, String name, Object handback)
         throws IllegalArgumentException {
@@ -610,6 +618,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
      * @exception ListenerNotFoundException if this listener is not
      *  registered in the MBean
      */
+    @Override
     public void removeAttributeChangeNotificationListener
         (NotificationListener listener, String name)
         throws ListenerNotFoundException {
@@ -659,6 +668,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
      * @exception RuntimeOperationsException wraps IllegalArgumentException
      *  when the specified notification is <code>null</code> or invalid
      */
+    @Override
     public void sendAttributeChangeNotification
         (AttributeChangeNotification notification)
         throws MBeanException, RuntimeOperationsException {
@@ -688,6 +698,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
      * @exception RuntimeOperationsException wraps IllegalArgumentException
      *  when the specified notification is <code>null</code> or invalid
      */
+    @Override
     public void sendAttributeChangeNotification
         (Attribute oldValue, Attribute newValue)
         throws MBeanException, RuntimeOperationsException {
@@ -712,8 +723,6 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
     }
 
 
-
-
     /**
      * Send a <code>Notification</code> to all registered listeners as a
      * <code>jmx.modelmbean.general</code> notification.
@@ -725,6 +734,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
      * @exception RuntimeOperationsException wraps IllegalArgumentException
      *  when the specified notification is <code>null</code> or invalid
      */
+    @Override
     public void sendNotification(Notification notification)
         throws MBeanException, RuntimeOperationsException {
 
@@ -750,6 +760,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
      * @exception RuntimeOperationsException wraps IllegalArgumentException
      *  when the specified notification is <code>null</code> or invalid
      */
+    @Override
     public void sendNotification(String message)
         throws MBeanException, RuntimeOperationsException {
 
@@ -764,8 +775,6 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
     }
 
 
-
-
     // ---------------------------------------- NotificationBroadcaster Methods
 
 
@@ -780,6 +789,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
      *
      * @exception IllegalArgumentException if the listener parameter is null
      */
+    @Override
     public void addNotificationListener(NotificationListener listener,
                                         NotificationFilter filter,
                                         Object handback)
@@ -814,6 +824,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
      * Return an <code>MBeanNotificationInfo</code> object describing the
      * notifications sent by this MBean.
      */
+    @Override
     public MBeanNotificationInfo[] getNotificationInfo() {
 
         // Acquire the set of application notifications
@@ -866,6 +877,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
      * @exception ListenerNotFoundException if this listener is not
      *  registered in the MBean
      */
+    @Override
     public void removeNotificationListener(NotificationListener listener)
         throws ListenerNotFoundException {
 
@@ -925,23 +937,23 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
     // ------------------------------------------------ PersistentMBean Methods
 
 
-    /**
-     * Instantiates this MBean instance from data found in the persistent
-     * store.  The data loaded could include attribute and operation values.
-     * This method should be called during construction or initialization
-     * of the instance, and before the MBean is registered with the
-     * <code>MBeanServer</code>.
-     *
-     * <p><strong>IMPLEMENTATION NOTE</strong> - This implementation does
-     * not support persistence.</p>
-     *
-     * @exception InstanceNotFoundException if the managed resource object
-     *  cannot be found
-     * @exception MBeanException if the initializer of the object throws
-     *  an exception
-     * @exception RuntimeOperationsException if an exception is reported
-     *  by the persistence mechanism
-     */
+//    /**
+//     * Instantiates this MBean instance from data found in the persistent
+//     * store.  The data loaded could include attribute and operation values.
+//     * This method should be called during construction or initialization
+//     * of the instance, and before the MBean is registered with the
+//     * <code>MBeanServer</code>.
+//     *
+//     * <p><strong>IMPLEMENTATION NOTE</strong> - This implementation does
+//     * not support persistence.</p>
+//     *
+//     * @exception InstanceNotFoundException if the managed resource object
+//     *  cannot be found
+//     * @exception MBeanException if the initializer of the object throws
+//     *  an exception
+//     * @exception RuntimeOperationsException if an exception is reported
+//     *  by the persistence mechanism
+//     */
 //    public void load() throws InstanceNotFoundException,
 //        MBeanException, RuntimeOperationsException {
 //        // XXX If a context was set, use it to load the data
@@ -952,22 +964,22 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
 //    }
 
 
-    /**
-     * Capture the current state of this MBean instance and write it out
-     * to the persistent store.  The state stored could include attribute
-     * and operation values.  If one of these methods of persistence is not
-     * supported, a "service not found" exception will be thrown.
-     *
-     * <p><strong>IMPLEMENTATION NOTE</strong> - This implementation does
-     * not support persistence.</p>
-     *
-     * @exception InstanceNotFoundException if the managed resource object
-     *  cannot be found
-     * @exception MBeanException if the initializer of the object throws
-     *  an exception, or persistence is not supported
-     * @exception RuntimeOperationsException if an exception is reported
-     *  by the persistence mechanism
-     */
+//    /**
+//     * Capture the current state of this MBean instance and write it out
+//     * to the persistent store.  The state stored could include attribute
+//     * and operation values.  If one of these methods of persistence is not
+//     * supported, a "service not found" exception will be thrown.
+//     *
+//     * <p><strong>IMPLEMENTATION NOTE</strong> - This implementation does
+//     * not support persistence.</p>
+//     *
+//     * @exception InstanceNotFoundException if the managed resource object
+//     *  cannot be found
+//     * @exception MBeanException if the initializer of the object throws
+//     *  an exception, or persistence is not supported
+//     * @exception RuntimeOperationsException if an exception is reported
+//     *  by the persistence mechanism
+//     */
 //    public void store() throws InstanceNotFoundException,
 //        MBeanException, RuntimeOperationsException {
 //
@@ -980,20 +992,20 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
 
     // --------------------  BaseModelMBean methods --------------------
 
-    /** Set the type of the mbean. This is used as a key to locate
-     * the description in the Registry.
-     *
-     * @param type the type of classname of the modeled object
-     */
+//    /** Set the type of the mbean. This is used as a key to locate
+//     * the description in the Registry.
+//     *
+//     * @param type the type of classname of the modeled object
+//     */
 //    void setModeledType( String type ) {
 //        initModelInfo(type);
 //        createResource();
 //    }
-    /** Set the type of the mbean. This is used as a key to locate
-     * the description in the Registry.
-     *
-     * @param type the type of classname of the modeled object
-     */
+//    /** Set the type of the mbean. This is used as a key to locate
+//     * the description in the Registry.
+//     *
+//     * @param type the type of classname of the modeled object
+//     */
 //    void initModelInfo( String type ) {
 //        try {
 //            if( log.isDebugEnabled())
@@ -1023,9 +1035,9 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
 //        }
 //    }
 
-    /** Set the type of the mbean. This is used as a key to locate
-     * the description in the Registry.
-     */
+//    /** Set the type of the mbean. This is used as a key to locate
+//     * the description in the Registry.
+//     */
 //    protected void createResource() {
 //        try {
 //            //Thread.currentThread().setContextClassLoader(BaseModelMBean.class.getClassLoader());
@@ -1078,9 +1090,9 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
     // ------------------------------------------------------ Protected Methods
 
 
-    /**
-     * Create and return a default <code>ModelMBeanInfo</code> object.
-     */
+//    /**
+//     * Create and return a default <code>ModelMBeanInfo</code> object.
+//     */
 //    protected ModelMBeanInfo createDefaultModelMBeanInfo() {
 //
 //        return (new ModelMBeanInfoSupport(this.getClass().getName(),
@@ -1089,15 +1101,15 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
 //
 //    }
 
-    /**
-     * Is the specified <code>ModelMBeanInfo</code> instance valid?
-     *
-     * <p><strong>IMPLEMENTATION NOTE</strong> - This implementation
-     * does not check anything, but this method can be overridden
-     * as required.</p>
-     *
-     * @param info The <code>ModelMBeanInfo object to check
-     */
+//    /**
+//     * Is the specified <code>ModelMBeanInfo</code> instance valid?
+//     *
+//     * <p><strong>IMPLEMENTATION NOTE</strong> - This implementation
+//     * does not check anything, but this method can be overridden
+//     * as required.</p>
+//     *
+//     * @param info The <code>ModelMBeanInfo object to check
+//     */
 //    protected boolean isModelMBeanInfoValid(ModelMBeanInfo info) {
 //        return (true);
 //    }
@@ -1106,6 +1118,7 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
     // XXX We can add some method patterns here- like setName() and
     // setDomain() for code that doesn't implement the Registration
 
+    @Override
     public ObjectName preRegister(MBeanServer server,
                                   ObjectName name)
             throws Exception
@@ -1119,18 +1132,21 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
         return oname;
     }
 
+    @Override
     public void postRegister(Boolean registrationDone) {
         if( resource instanceof MBeanRegistration ) {
             ((MBeanRegistration)resource).postRegister(registrationDone);
         }
     }
 
+    @Override
     public void preDeregister() throws Exception {
         if( resource instanceof MBeanRegistration ) {
             ((MBeanRegistration)resource).preDeregister();
         }
     }
 
+    @Override
     public void postDeregister() {
         if( resource instanceof MBeanRegistration ) {
             ((MBeanRegistration)resource).postDeregister();
@@ -1138,38 +1154,40 @@ public class BaseModelMBean implements DynamicMBean, MBeanRegistration, ModelMBe
     }
 
     static class MethodKey {
-	private String name;
-	private String[] signature;
+        private String name;
+        private String[] signature;
 
-	MethodKey(String name, String[] signature) {
-	    this.name = name;
-	    if(signature == null) {
-		signature = new String[0];
-	    }
-	    this.signature = signature;
-	}
+        MethodKey(String name, String[] signature) {
+            this.name = name;
+            if(signature == null) {
+                signature = new String[0];
+            }
+            this.signature = signature;
+        }
 
-	public boolean equals(Object other) {
-	    if(!(other instanceof MethodKey)) {
-		return false;
-	    }
-	    MethodKey omk = (MethodKey)other;
-	    if(!name.equals(omk.name)) {
-		return false;
-	    }
-	    if(signature.length != omk.signature.length) {
-		return false;
-	    }
-	    for(int i=0; i < signature.length; i++) {
-		if(!signature[i].equals(omk.signature[i])) {
-		    return false;
-		}
-	    }
-	    return true;
-	}
+        @Override
+        public boolean equals(Object other) {
+            if(!(other instanceof MethodKey)) {
+                return false;
+            }
+            MethodKey omk = (MethodKey)other;
+            if(!name.equals(omk.name)) {
+                return false;
+            }
+            if(signature.length != omk.signature.length) {
+                return false;
+            }
+            for(int i=0; i < signature.length; i++) {
+                if(!signature[i].equals(omk.signature[i])) {
+                    return false;
+                }
+            }
+            return true;
+        }
 
-	public int hashCode() {
-	    return name.hashCode();
-	}
+        @Override
+        public int hashCode() {
+            return name.hashCode();
+        }
     }
 }
