@@ -29,7 +29,7 @@ import java.net.SocketException;
 import java.security.KeyManagementException;
 import java.security.KeyStore;
 import java.security.NoSuchAlgorithmException;
-import java.security.SecureRandom;
+import java.security.UnrecoverableKeyException;
 import java.security.cert.CRL;
 import java.security.cert.CRLException;
 import java.security.cert.CertPathParameters;
@@ -41,6 +41,7 @@ import java.security.cert.CollectionCertStoreParameters;
 import java.security.cert.PKIXBuilderParameters;
 import java.security.cert.X509CertSelector;
 import java.util.Collection;
+import java.util.Locale;
 import java.util.Vector;
 
 import javax.net.ssl.CertPathTrustManagerParameters;
@@ -51,26 +52,25 @@ import javax.net.ssl.SSLContext;
 import javax.net.ssl.SSLException;
 import javax.net.ssl.SSLServerSocket;
 import javax.net.ssl.SSLServerSocketFactory;
+import javax.net.ssl.SSLSession;
 import javax.net.ssl.SSLSessionContext;
 import javax.net.ssl.SSLSocket;
 import javax.net.ssl.TrustManager;
 import javax.net.ssl.TrustManagerFactory;
 import javax.net.ssl.X509KeyManager;
 
+import org.apache.tomcat.util.net.AbstractEndpoint;
+import org.apache.tomcat.util.net.Constants;
+import org.apache.tomcat.util.net.SSLUtil;
+import org.apache.tomcat.util.net.ServerSocketFactory;
 import org.apache.tomcat.util.res.StringManager;
 
-/*
-  1. Make the JSSE's jars available, either as an installed
-     extension (copy them into jre/lib/ext) or by adding
-     them to the Tomcat classpath.
-  2. keytool -genkey -alias tomcat -keyalg RSA
-     Use "changeit" as password ( this is the default we use )
- */
-
 /**
- * SSL server socket factory. It _requires_ a valid RSA key and
- * JSSE. 
- *
+ * SSL server socket factory. It <b>requires</b> a valid RSA key and
+ * JSSE.<br/>
+ * keytool -genkey -alias tomcat -keyalg RSA</br>
+ * Use "changeit" as password (this is the default we use).
+ * 
  * @author Harish Prabandham
  * @author Costin Manolache
  * @author Stefan Freyr Stefansson
@@ -78,33 +78,31 @@ import org.apache.tomcat.util.res.StringManager;
  * @author Jan Luehe
  * @author Bill Barker
  */
-public class JSSESocketFactory
-    extends org.apache.tomcat.util.net.ServerSocketFactory {
+public class JSSESocketFactory implements ServerSocketFactory, SSLUtil {
 
-    private static StringManager sm =
+    private static final org.apache.juli.logging.Log log =
+        org.apache.juli.logging.LogFactory.getLog(JSSESocketFactory.class);
+    private static final StringManager sm =
         StringManager.getManager("org.apache.tomcat.util.net.jsse.res");
 
     private static final boolean RFC_5746_SUPPORTED;
 
-    // defaults
-    static String defaultProtocol = "TLS";
-    static boolean defaultClientAuth = false;
-    static String defaultKeystoreType = "JKS";
+    // Defaults - made public where re-used
+    private static final String defaultProtocol = "TLS";
+    private static final String defaultKeystoreType = "JKS";
     private static final String defaultKeystoreFile
         = System.getProperty("user.home") + "/.keystore";
-    private static final String defaultKeyPass = "changeit";
     private static final int defaultSessionCacheSize = 0;
     private static final int defaultSessionTimeout = 86400;
+    private static final String ALLOW_ALL_SUPPORTED_CIPHERS = "ALL";
+    public static final String DEFAULT_KEY_PASS = "changeit";
     
-    static org.apache.juli.logging.Log log =
-        org.apache.juli.logging.LogFactory.getLog(JSSESocketFactory.class);
-
     static {
         boolean result = false;
         SSLContext context;
         try {
             context = SSLContext.getInstance("TLS");
-            context.init(null, null, new SecureRandom());
+            context.init(null, null, null);
             SSLServerSocketFactory ssf = context.getServerSocketFactory();
             String ciphers[] = ssf.getSupportedCipherSuites();
             for (String cipher : ciphers) {
@@ -121,8 +119,9 @@ public class JSSESocketFactory
         RFC_5746_SUPPORTED = result;
     }
 
-    protected boolean initialized;
-    protected String clientAuth = "false";
+
+    private AbstractEndpoint endpoint;
+
     protected SSLServerSocketFactory sslProxy = null;
     protected String[] enabledCiphers;
     protected boolean allowUnsafeLegacyRenegotiation = false;
@@ -138,53 +137,61 @@ public class JSSESocketFactory
     protected boolean wantClientAuth    = false;
 
 
-    public JSSESocketFactory () {
+    public JSSESocketFactory (AbstractEndpoint endpoint) {
+        this.endpoint = endpoint;
     }
 
+    @Override
     public ServerSocket createSocket (int port)
         throws IOException
     {
-        if (!initialized) init();
+        init();
         ServerSocket socket = sslProxy.createServerSocket(port);
         initServerSocket(socket);
         return socket;
     }
     
+    @Override
     public ServerSocket createSocket (int port, int backlog)
         throws IOException
     {
-        if (!initialized) init();
+        init();
         ServerSocket socket = sslProxy.createServerSocket(port, backlog);
         initServerSocket(socket);
         return socket;
     }
     
+    @Override
     public ServerSocket createSocket (int port, int backlog,
                                       InetAddress ifAddress)
         throws IOException
     {   
-        if (!initialized) init();
+        init();
         ServerSocket socket = sslProxy.createServerSocket(port, backlog,
                                                           ifAddress);
         initServerSocket(socket);
         return socket;
     }
     
+    @Override
     public Socket acceptSocket(ServerSocket socket)
         throws IOException
     {
         SSLSocket asock = null;
         try {
              asock = (SSLSocket)socket.accept();
-             configureClientAuth(asock);
         } catch (SSLException e){
           throw new SocketException("SSL handshake error" + e.toString());
         }
         return asock;
     }
     
+    @Override
     public void handshake(Socket sock) throws IOException {
-        ((SSLSocket)sock).startHandshake();
+        // We do getSession instead of startHandshake() so we can call this multiple times
+        SSLSession session = ((SSLSocket)sock).getSession();
+        if (session.getCipherSuite().equals("SSL_NULL_WITH_NULL_NULL"))
+            throw new IOException("SSL handshake failed. Ciper suite in SSL Session is SSL_NULL_WITH_NULL_NULL");
 
         if (!allowUnsafeLegacyRenegotiation && !RFC_5746_SUPPORTED) {
             // Prevent further handshakes by removing all cipher suites
@@ -204,16 +211,21 @@ public class JSSESocketFactory
     protected String[] getEnabledCiphers(String requestedCiphers,
                                          String[] supportedCiphers) {
 
-        String[] enabledCiphers = null;
+        String[] result = null;
+
+        if (ALLOW_ALL_SUPPORTED_CIPHERS.equals(requestedCiphers)) {
+            return supportedCiphers;
+        }
 
         if (requestedCiphers != null) {
-            Vector vec = null;
+            Vector<String> vec = null;
             String cipher = requestedCiphers;
             int index = requestedCiphers.indexOf(',');
             if (index != -1) {
                 int fromIndex = 0;
                 while (index != -1) {
-                    cipher = requestedCiphers.substring(fromIndex, index).trim();
+                    cipher =
+                        requestedCiphers.substring(fromIndex, index).trim();
                     if (cipher.length() > 0) {
                         /*
                          * Check to see if the requested cipher is among the
@@ -223,7 +235,7 @@ public class JSSESocketFactory
                                      && i<supportedCiphers.length; i++) {
                             if (supportedCiphers[i].equals(cipher)) {
                                 if (vec == null) {
-                                    vec = new Vector();
+                                    vec = new Vector<String>();
                                 }
                                 vec.addElement(cipher);
                                 break;
@@ -247,7 +259,7 @@ public class JSSESocketFactory
                                  && i<supportedCiphers.length; i++) {
                         if (supportedCiphers[i].equals(cipher)) {
                             if (vec == null) {
-                                vec = new Vector();
+                                vec = new Vector<String>();
                             }
                             vec.addElement(cipher);
                             break;
@@ -257,25 +269,25 @@ public class JSSESocketFactory
             }           
 
             if (vec != null) {
-                enabledCiphers = new String[vec.size()];
-                vec.copyInto(enabledCiphers);
+                result = new String[vec.size()];
+                vec.copyInto(result);
             }
         } else {
-            enabledCiphers = sslProxy.getDefaultCipherSuites();
+            result = sslProxy.getDefaultCipherSuites();
         }
 
-        return enabledCiphers;
+        return result;
     }
      
     /*
      * Gets the SSL server's keystore password.
      */
     protected String getKeystorePassword() {
-        String keyPass = (String)attributes.get("keypass");
+        String keyPass = endpoint.getKeyPass();
         if (keyPass == null) {
-            keyPass = defaultKeyPass;
+            keyPass = DEFAULT_KEY_PASS;
         }
-        String keystorePass = (String)attributes.get("keystorePass");
+        String keystorePass = endpoint.getKeystorePass();
         if (keystorePass == null) {
             keystorePass = keyPass;
         }
@@ -288,19 +300,11 @@ public class JSSESocketFactory
     protected KeyStore getKeystore(String type, String provider, String pass)
             throws IOException {
 
-        String keystoreFile = (String)attributes.get("keystore");
+        String keystoreFile = endpoint.getKeystoreFile();
         if (keystoreFile == null)
             keystoreFile = defaultKeystoreFile;
 
-        try {
-            return getStore(type, provider, keystoreFile, pass);
-        } catch (FileNotFoundException fnfe) {
-            throw fnfe;
-        } catch (IOException ioe) {
-            log.error(sm.getString("jsse.keystore_load_failed", type,
-                    keystoreFile, ioe.getMessage()), ioe);
-            throw ioe;
-        }
+        return getStore(type, provider, keystoreFile, pass);
     }
 
     /*
@@ -310,24 +314,24 @@ public class JSSESocketFactory
             String keystoreProvider) throws IOException {
         KeyStore trustStore = null;
 
-        String truststoreFile = (String)attributes.get("truststoreFile");
+        String truststoreFile = endpoint.getTruststoreFile();
         if(truststoreFile == null) {
             truststoreFile = System.getProperty("javax.net.ssl.trustStore");
         }
         if(log.isDebugEnabled()) {
             log.debug("Truststore = " + truststoreFile);
         }
-        String truststorePassword = (String)attributes.get("truststorePass");
+
+        String truststorePassword = endpoint.getTruststorePass();
         if( truststorePassword == null) {
-            truststorePassword = System.getProperty("javax.net.ssl.trustStorePassword");
-        }
-        if( truststorePassword == null ) {
-            truststorePassword = getKeystorePassword();
+            truststorePassword =
+                System.getProperty("javax.net.ssl.trustStorePassword");
         }
         if(log.isDebugEnabled()) {
             log.debug("TrustPass = " + truststorePassword);
         }
-        String truststoreType = (String)attributes.get("truststoreType");
+
+        String truststoreType = endpoint.getTruststoreType();
         if( truststoreType == null) {
             truststoreType = System.getProperty("javax.net.ssl.trustStoreType");
         }
@@ -337,8 +341,8 @@ public class JSSESocketFactory
         if(log.isDebugEnabled()) {
             log.debug("trustType = " + truststoreType);
         }
-        String truststoreProvider =
-            (String)attributes.get("truststoreProvider");
+
+        String truststoreProvider = endpoint.getTruststoreProvider();
         if( truststoreProvider == null) {
             truststoreProvider =
                 System.getProperty("javax.net.ssl.trustStoreProvider");
@@ -350,30 +354,21 @@ public class JSSESocketFactory
             log.debug("trustProvider = " + truststoreProvider);
         }
 
-        if (truststoreFile != null) {
+        if (truststoreFile != null){
             try {
                 trustStore = getStore(truststoreType, truststoreProvider,
                         truststoreFile, truststorePassword);
-            } catch (FileNotFoundException fnfe) {
-                throw fnfe;
             } catch (IOException ioe) {
-                // Log a warning that we had a password issue
-                // and re-try, unless the password is null already
-                if (truststorePassword != null) {
+                Throwable cause = ioe.getCause();
+                if (cause instanceof UnrecoverableKeyException) {
+                    // Log a warning we had a password issue
                     log.warn(sm.getString("jsse.invalid_truststore_password"),
-                            ioe);
-                    try {
-                        trustStore = getStore(truststoreType,
-                                truststoreProvider, truststoreFile, null);
-                        ioe = null;
-                    } catch (IOException ioe2) {
-                        ioe = ioe2;
-                    }
-                }
-                if (ioe != null) {
-                    log.error(sm.getString("jsse.keystore_load_failed",
-                            truststoreType, truststoreFile, ioe.getMessage()),
-                            ioe);
+                            cause);
+                    // Re-try
+                    trustStore = getStore(truststoreType, truststoreProvider,
+                            truststoreFile, null);
+                } else {
+                    // Something else went wrong - re-throw
                     throw ioe;
                 }
             }
@@ -396,18 +391,19 @@ public class JSSESocketFactory
             } else {
                 ks = KeyStore.getInstance(type, provider);
             }
-            if(!("PKCS11".equalsIgnoreCase(type) || "".equalsIgnoreCase(path))) {
+            if(!("PKCS11".equalsIgnoreCase(type) ||
+                    "".equalsIgnoreCase(path))) {
                 File keyStoreFile = new File(path);
                 if (!keyStoreFile.isAbsolute()) {
-                    keyStoreFile = new File(System.getProperty("catalina.base"),
-                                            path);
+                    keyStoreFile = new File(System.getProperty(
+                            Constants.CATALINA_BASE_PROP), path);
                 }
                 istream = new FileInputStream(keyStoreFile);
             }
-
+            
             char[] storePass = null;
             if (pass != null && !"".equals(pass)) {
-                storePass = pass.toCharArray();
+                storePass = pass.toCharArray(); 
             }
             ks.load(istream, storePass);
         } catch (FileNotFoundException fnfe) {
@@ -442,7 +438,7 @@ public class JSSESocketFactory
     void init() throws IOException {
         try {
 
-            String clientAuthStr = (String) attributes.get("clientauth");
+            String clientAuthStr = endpoint.getClientAuth();
             if("true".equalsIgnoreCase(clientAuthStr) ||
                "yes".equalsIgnoreCase(clientAuthStr)) {
                 requireClientAuth = true;
@@ -450,73 +446,26 @@ public class JSSESocketFactory
                 wantClientAuth = true;
             }
 
-            // SSL protocol variant (e.g., TLS, SSL v3, etc.)
-            String protocol = (String) attributes.get("protocol");
-            if (protocol == null) {
-                protocol = defaultProtocol;
-            }
-
-            // Certificate encoding algorithm (e.g., SunX509)
-            String algorithm = (String) attributes.get("algorithm");
-            if (algorithm == null) {
-                algorithm = KeyManagerFactory.getDefaultAlgorithm();;
-            }
-
-            String keystoreType = (String) attributes.get("keystoreType");
-            if (keystoreType == null) {
-                keystoreType = defaultKeystoreType;
-            }
-
-            String keystoreProvider =
-                (String) attributes.get("keystoreProvider");
-
-            String trustAlgorithm =
-                (String)attributes.get("truststoreAlgorithm");
-            if( trustAlgorithm == null ) {
-                trustAlgorithm = TrustManagerFactory.getDefaultAlgorithm();
-            }
-
-            // Create and init SSLContext
-            SSLContext context = SSLContext.getInstance(protocol); 
-            context.init(getKeyManagers(keystoreType, keystoreProvider,
-                                 algorithm,
-                                 (String) attributes.get("keyAlias")),
-                         getTrustManagers(keystoreType, keystoreProvider,
-                                 trustAlgorithm),
-                         new SecureRandom());
+            SSLContext context = createSSLContext();
+            context.init(getKeyManagers(), getTrustManagers(), null);
 
             // Configure SSL session cache
-            int sessionCacheSize;
-            if (attributes.get("sessionCacheSize") != null) {
-                sessionCacheSize = Integer.parseInt(
-                        (String)attributes.get("sessionCacheSize"));
-            } else {
-                sessionCacheSize = defaultSessionCacheSize;
-            }
-            int sessionTimeout;
-            if (attributes.get("sessionTimeout") != null) {
-                sessionTimeout = Integer.parseInt(
-                        (String)attributes.get("sessionTimeout"));
-            } else {
-                sessionTimeout = defaultSessionTimeout;
-            }
             SSLSessionContext sessionContext =
                 context.getServerSessionContext();
             if (sessionContext != null) {
-                sessionContext.setSessionCacheSize(sessionCacheSize);
-                sessionContext.setSessionTimeout(sessionTimeout);
+                configureSessionContext(sessionContext);
             }
 
             // create proxy
             sslProxy = context.getServerSocketFactory();
 
             // Determine which cipher suites to enable
-            String requestedCiphers = (String)attributes.get("ciphers");
+            String requestedCiphers = endpoint.getCiphers();
             enabledCiphers = getEnabledCiphers(requestedCiphers,
-                                               sslProxy.getSupportedCipherSuites());
+                    sslProxy.getSupportedCipherSuites());
 
-            allowUnsafeLegacyRenegotiation =
-                "true".equals(attributes.get("allowUnsafeLegacyRenegotiation"));
+            allowUnsafeLegacyRenegotiation = "true".equals(
+                    endpoint.getAllowUnsafeLegacyRenegotiation());
             
             // Check the SSL config is OK
             checkConfig();
@@ -524,10 +473,77 @@ public class JSSESocketFactory
         } catch(Exception e) {
             if( e instanceof IOException )
                 throw (IOException)e;
-            throw new IOException(e.getMessage());
+            throw new IOException(e.getMessage(), e);
         }
     }
 
+    @Override
+    public SSLContext createSSLContext() throws Exception {
+
+        // SSL protocol variant (e.g., TLS, SSL v3, etc.)
+        String protocol = endpoint.getSslProtocol();
+        if (protocol == null) {
+            protocol = defaultProtocol;
+        }
+
+        SSLContext context = SSLContext.getInstance(protocol); 
+
+        return context;
+    }
+    
+    @Override
+    public KeyManager[] getKeyManagers() throws Exception {
+        String keystoreType = endpoint.getKeystoreType();
+        if (keystoreType == null) {
+            keystoreType = defaultKeystoreType;
+        }
+
+        String algorithm = endpoint.getAlgorithm();
+        if (algorithm == null) {
+            algorithm = KeyManagerFactory.getDefaultAlgorithm();
+        }
+
+        return getKeyManagers(keystoreType, endpoint.getKeystoreProvider(),
+                algorithm, endpoint.getKeyAlias());
+    }
+
+    @Override
+    public TrustManager[] getTrustManagers() throws Exception {
+        String keystoreType = endpoint.getKeystoreType();
+        if (keystoreType == null) {
+            keystoreType = defaultKeystoreType;
+        }
+
+        String algorithm = endpoint.getAlgorithm();
+        if (algorithm == null) {
+            algorithm = KeyManagerFactory.getDefaultAlgorithm();
+        }
+
+        return getTrustManagers(keystoreType, endpoint.getKeystoreProvider(),
+                algorithm);
+    }
+
+    @Override
+    public void configureSessionContext(SSLSessionContext sslSessionContext) {
+        int sessionCacheSize;
+        if (endpoint.getSessionCacheSize() != null) {
+            sessionCacheSize = Integer.parseInt(
+                    endpoint.getSessionCacheSize());
+        } else {
+            sessionCacheSize = defaultSessionCacheSize;
+        }
+        
+        int sessionTimeout;
+        if (endpoint.getSessionTimeout() != null) {
+            sessionTimeout = Integer.parseInt(endpoint.getSessionTimeout());
+        } else {
+            sessionTimeout = defaultSessionTimeout;
+        }
+
+        sslSessionContext.setSessionCacheSize(sessionCacheSize);
+        sslSessionContext.setSessionTimeout(sessionTimeout);
+    }
+
     /**
      * Gets the initialized key managers.
      */
@@ -543,19 +559,25 @@ public class JSSESocketFactory
 
         KeyStore ks = getKeystore(keystoreType, keystoreProvider, keystorePass);
         if (keyAlias != null && !ks.isKeyEntry(keyAlias)) {
-            throw new IOException(sm.getString("jsse.alias_no_key_entry", keyAlias));
+            throw new IOException(
+                    sm.getString("jsse.alias_no_key_entry", keyAlias));
         }
 
         KeyManagerFactory kmf = KeyManagerFactory.getInstance(algorithm);
-        kmf.init(ks, keystorePass.toCharArray());
+        String keyPass = endpoint.getKeyPass();
+        if (keyPass == null) {
+            keyPass = keystorePass;
+        }
+        kmf.init(ks, keyPass.toCharArray());
 
         kms = kmf.getKeyManagers();
         if (keyAlias != null) {
+            String alias = keyAlias;
             if (JSSESocketFactory.defaultKeystoreType.equals(keystoreType)) {
-                keyAlias = keyAlias.toLowerCase();
+                alias = alias.toLowerCase(Locale.ENGLISH);
             }
             for(int i=0; i<kms.length; i++) {
-                kms[i] = new JSSEKeyManager((X509KeyManager)kms[i], keyAlias);
+                kms[i] = new JSSEKeyManager((X509KeyManager)kms[i], alias);
             }
         }
 
@@ -563,25 +585,42 @@ public class JSSESocketFactory
     }
 
     /**
-     * Gets the intialized trust managers.
+     * Gets the initialized trust managers.
      */
     protected TrustManager[] getTrustManagers(String keystoreType,
             String keystoreProvider, String algorithm)
         throws Exception {
-        String crlf = (String) attributes.get("crlFile");
+        String crlf = endpoint.getCrlFile();
         
+        String className = endpoint.getTrustManagerClassName();
+        if(className != null && className.length() > 0) {
+             ClassLoader classLoader = getClass().getClassLoader();
+             Class<?> clazz = classLoader.loadClass(className);
+             if(!(TrustManager.class.isAssignableFrom(clazz))){
+                throw new InstantiationException(sm.getString(
+                        "jsse.invalidTrustManagerClassName", className));
+             }
+             Object trustManagerObject = clazz.newInstance();
+             TrustManager trustManager = (TrustManager) trustManagerObject;
+             return new TrustManager[]{ trustManager };
+        }    
+
         TrustManager[] tms = null;
         
         KeyStore trustStore = getTrustStore(keystoreType, keystoreProvider);
-        if (trustStore != null) {
+        if (trustStore != null || endpoint.getTrustManagerClassName() != null) {
             if (crlf == null) {
-                TrustManagerFactory tmf = TrustManagerFactory.getInstance(algorithm);
+                TrustManagerFactory tmf =
+                    TrustManagerFactory.getInstance(algorithm);
                 tmf.init(trustStore);
                 tms = tmf.getTrustManagers();
             } else {
-                TrustManagerFactory tmf = TrustManagerFactory.getInstance(algorithm);
-                CertPathParameters params = getParameters(algorithm, crlf, trustStore);
-                ManagerFactoryParameters mfp = new CertPathTrustManagerParameters(params);
+                TrustManagerFactory tmf =
+                    TrustManagerFactory.getInstance(algorithm);
+                CertPathParameters params =
+                    getParameters(algorithm, crlf, trustStore);
+                ManagerFactoryParameters mfp =
+                    new CertPathTrustManagerParameters(params);
                 tmf.init(mfp);
                 tms = tmf.getTrustManagers();
             }
@@ -589,7 +628,7 @@ public class JSSESocketFactory
         
         return tms;
     }
-    
+
     /**
      * Return the initialization parameters for the TrustManager.
      * Currently, only the default <code>PKIX</code> is supported.
@@ -605,14 +644,14 @@ public class JSSESocketFactory
         throws Exception {
         CertPathParameters params = null;
         if("PKIX".equalsIgnoreCase(algorithm)) {
-            PKIXBuilderParameters xparams = new PKIXBuilderParameters(trustStore, 
-                                                                     new X509CertSelector());
-            Collection crls = getCRLs(crlf);
+            PKIXBuilderParameters xparams =
+                new PKIXBuilderParameters(trustStore, new X509CertSelector());
+            Collection<? extends CRL> crls = getCRLs(crlf);
             CertStoreParameters csp = new CollectionCertStoreParameters(crls);
             CertStore store = CertStore.getInstance("Collection", csp);
             xparams.addCertStore(store);
             xparams.setRevocationEnabled(true);
-            String trustLength = (String)attributes.get("trustMaxCertLength");
+            String trustLength = endpoint.getTrustMaxCertLength();
             if(trustLength != null) {
                 try {
                     xparams.setMaxPathLength(Integer.parseInt(trustLength));
@@ -638,7 +677,8 @@ public class JSSESocketFactory
 
         File crlFile = new File(crlf);
         if( !crlFile.isAbsolute() ) {
-            crlFile = new File(System.getProperty("catalina.base"), crlf);
+            crlFile = new File(
+                    System.getProperty(Constants.CATALINA_BASE_PROP), crlf);
         }
         Collection<? extends CRL> crls = null;
         InputStream is = null;
@@ -657,6 +697,7 @@ public class JSSESocketFactory
                 try{
                     is.close();
                 } catch(Exception ex) {
+                    // Ignore
                 }
             }
         }
@@ -668,7 +709,8 @@ public class JSSESocketFactory
      * @param socket the SSLServerSocket.
      * @param protocols the protocols to use.
      */
-    protected void setEnabledProtocols(SSLServerSocket socket, String []protocols){
+    protected void setEnabledProtocols(SSLServerSocket socket,
+            String []protocols){
         if (protocols != null) {
             socket.setEnabledProtocols(protocols);
         }
@@ -678,67 +720,36 @@ public class JSSESocketFactory
      * Determines the SSL protocol variants to be enabled.
      *
      * @param socket The socket to get supported list from.
-     * @param requestedProtocols Comma-separated list of requested SSL
-     * protocol variants
+     * @param requestedProtocols Array of requested protocol names all of which
+     *                           must be non-null and non-zero length
      *
      * @return Array of SSL protocol variants to be enabled, or null if none of
      * the requested protocol variants are supported
      */
     protected String[] getEnabledProtocols(SSLServerSocket socket,
-                                           String requestedProtocols){
+                                           String[] requestedProtocols){
         String[] supportedProtocols = socket.getSupportedProtocols();
 
         String[] enabledProtocols = null;
 
-        if (requestedProtocols != null) {
-            Vector vec = null;
-            String protocol = requestedProtocols;
-            int index = requestedProtocols.indexOf(',');
-            if (index != -1) {
-                int fromIndex = 0;
-                while (index != -1) {
-                    protocol = requestedProtocols.substring(fromIndex, index).trim();
-                    if (protocol.length() > 0) {
-                        /*
-                         * Check to see if the requested protocol is among the
-                         * supported protocols, i.e., may be enabled
-                         */
-                        for (int i=0; supportedProtocols != null
-                                     && i<supportedProtocols.length; i++) {
-                            if (supportedProtocols[i].equals(protocol)) {
-                                if (vec == null) {
-                                    vec = new Vector();
-                                }
-                                vec.addElement(protocol);
-                                break;
-                            }
-                        }
-                    }
-                    fromIndex = index+1;
-                    index = requestedProtocols.indexOf(',', fromIndex);
-                } // while
-                protocol = requestedProtocols.substring(fromIndex);
-            }
-
-            if (protocol != null) {
-                protocol = protocol.trim();
-                if (protocol.length() > 0) {
-                    /*
-                     * Check to see if the requested protocol is among the
-                     * supported protocols, i.e., may be enabled
-                     */
-                    for (int i=0; supportedProtocols != null
-                                 && i<supportedProtocols.length; i++) {
-                        if (supportedProtocols[i].equals(protocol)) {
-                            if (vec == null) {
-                                vec = new Vector();
-                            }
-                            vec.addElement(protocol);
-                            break;
+        if (requestedProtocols != null && requestedProtocols.length > 0) {
+            Vector<String> vec = null;
+            for (String protocol : requestedProtocols) {
+                /*
+                 * Check to see if the requested protocol is among the supported
+                 * protocols, i.e., may be enabled
+                 */
+                for (int i=0; supportedProtocols != null &&
+                        i < supportedProtocols.length; i++) {
+                    if (supportedProtocols[i].equals(protocol)) {
+                        if (vec == null) {
+                            vec = new Vector<String>();
                         }
+                        vec.addElement(protocol);
+                        break;
                     }
                 }
-            }           
+            }
 
             if (vec != null) {
                 enabledProtocols = new String[vec.size()];
@@ -763,17 +774,6 @@ public class JSSESocketFactory
         }
     }
 
-    /**
-     * Configure Client authentication for this version of JSSE.  The
-     * JSSE included in Java 1.4 supports the 'want' value.  Prior
-     * versions of JSSE will treat 'want' as 'false'.
-     * @param socket the SSLSocket
-     */
-    protected void configureClientAuth(SSLSocket socket){
-        // Per JavaDocs: SSLSockets returned from 
-        // SSLServerSocket.accept() inherit this setting.
-    }
-    
     /**
      * Configures the given SSL server socket with the requested cipher suites,
      * protocol versions, and need for client authentication
@@ -786,7 +786,7 @@ public class JSSESocketFactory
             socket.setEnabledCipherSuites(enabledCiphers);
         }
 
-        String requestedProtocols = (String) attributes.get("protocols");
+        String[] requestedProtocols = endpoint.getSslEnabledProtocolsArray();
         setEnabledProtocols(socket, getEnabledProtocols(socket, 
                                                          requestedProtocols));
 
